import json
from collections import OrderedDict
from wsgiref import simple_server

import falcon
from falcon_multipart.middleware import MultipartMiddleware
from oslo_config import cfg
from oslo_log import log
from oslo_policy import policy
from paste.deploy import loadapp
from tveapi import apps
from tveapi import policy as tveapi_policy
from tveapi import v1
from tveapi.common import exceptions as tveapi_exc
from tveapi.common import middlewares
from tveapi.common import utils
from tveapi.token import auth_token
import os

LOG = log.getLogger(__name__)
CONF = cfg.CONF
documentation = OrderedDict()
documentation['404'] = ("The API call you tried to make was not defined. "
                        "Here's a definition of the API to help you get going :)")
IP = "0.0.0.0"
PORT = 7373
PORT_AUTH_TOKEN_APP = 7374
tve_app = falcon.API()
tve_app_auth_token = falcon.API()
_already_parsed = False

def parse_args():
    global _already_parsed
    if not _already_parsed:
        log.register_options(CONF)
        policy.Enforcer(CONF)
        default_config_files = cfg.find_config_files('tveapi')
        CONF(
            default_config_files=default_config_files,
        )


def get_all_routes(api):
    routes_list = []
    def get_children(node):
        if len(node.children):
            for child_node in node.children:
                get_children(child_node)
        else:
            routes_list.append((node.uri_template, node.resource))
    [get_children(node) for node in api._router._roots]
    return routes_list


def create_documentation(app):
    doc_content = OrderedDict()
    version_dict = OrderedDict()
    if not app:
        return doc_content
    for method, value in get_all_routes(app):
        try:
            doc = version_dict.setdefault(method, OrderedDict())
            doc['endpoint'] = '{}{}:{}{}'.format('http://', IP, PORT, method)
        except AttributeError as e:
            LOG.error(f"Skipping entry due to AttributeError: {e}")

    doc_content['handlers'] = version_dict
    return doc_content


def app_factory(global_config, **local_config):
    tveapi_policy.setup_policy(CONF)

    middleware_list = [utils.FuncMiddleware(hook) for hook in
                       utils.before_hooks()]
    middleware_list.append(middlewares.RequireJSON())
    middleware_list.append(middlewares.JSONTranslator())
    middleware_list.append(middlewares.CustomAuthMiddleware())
    middleware_list.append(MultipartMiddleware())
    tve_app = falcon.API(middleware=middleware_list)

    for exception_class in tveapi_exc.exception_handlers_catalog:
        tve_app.add_error_handler(exception_class, exception_class.handle)

    endpoint_catalog = [
        ('/v1', v1.public_endpoints()),
        ('/apps', apps.public_endpoints()),
    ]
    for version_path, endpoints in endpoint_catalog:
        for route, resource in endpoints:
            tve_app.add_route(version_path + route, resource)

    tve_app.add_sink(handle_404, '/')
    documentation['documentation'] = create_documentation(tve_app);
    return tve_app


def app_factory_auth_token(global_config, **local_config):
    tveapi_policy.setup_policy(CONF)

    tve_app_auth_token = falcon.API()

    for exception_class in tveapi_exc.exception_handlers_catalog:
        tve_app_auth_token.add_error_handler(exception_class, exception_class.handle)

    tve_app_auth_token.add_route('/v3/auth/tokens', auth_token())

    tve_app_auth_token.add_sink(handle_404, '/')
    documentation['documentation'] = create_documentation(tve_app_auth_token);
    return tve_app_auth_token
    

def handle_404(req, resp):
    resp.status = falcon.HTTP_404
    resp.data = json.dumps(documentation, indent=4, separators=(',', ': ')).encode('utf8')
    resp.content_type = 'application/json'


def initialize_app():
    #     LOG.info("STARTING THE TVE API")
    paste_config_file = "/etc/tveapi/paste.ini"
    parse_args()
    app = loadapp('config:%s' % paste_config_file)
    return app
    

def initialize_auth_token_app():
    LOG.info("STARTING AUTH TOKEN SERVER!")
    paste_config_file = "/etc/tveapi/paste_auth_token.ini"
    parse_args()
    app = loadapp('config:%s' % paste_config_file)
    return app


def setup_logging():
    _DEFAULT_LOG_LEVELS = ['oslo_log=DEBUG',
                           'iso8601=WARN', 'requests.packages.urllib3.connectionpool=WARN',
                           'urllib3.connectionpool=WARN', 'websocket=WARN',
                           'keystonemiddleware=WARN', 'routes.middleware=WARN']
    _DEFAULT_LOGGING_CONTEXT_FORMAT = ('%(asctime)s.%(msecs)03d %(process)d '
                                       '%(levelname)s %(name)s [%(request_id)s'
                                       ' %(user_identity)s] %(instance)s '
                                       '%(message)s')
    log.set_defaults(_DEFAULT_LOGGING_CONTEXT_FORMAT, _DEFAULT_LOG_LEVELS)
    log.setup(CONF, 'tveapi', 'v1')
    log.setup(CONF, 'tveapi', 'apps')


def main():
    parse_args()
    setup_logging()
    
    paste_config_file = "/etc/tveapi/paste.ini"
    application = app = loadapp('config:%s' % paste_config_file)
    http = simple_server.make_server(IP, PORT, app)
    http.serve_forever()
    
    paste_config_file = "/etc/tveapi/paste_auth_token.ini"
    application_auth_token = app_auth_token = loadapp('config:%s' % paste_config_file)
    http_token = simple_server.make_server(IP, PORT_AUTH_TOKEN_APP, app_auth_token)
    http_token.serve_forever()


if __name__ == "__main__":
    main()
