#!/usr/bin/env python3
import falcon
import json
import os
import subprocess
import sys
import uuid
from collections import OrderedDict
from oslo_log import log
from tveapi.common import resource as res

sys.path.insert(1, '/usr/lib/python3/dist-packages/tveapi/')
import send_notification

LOG = log.getLogger(__name__)


class Applications(object):
    @staticmethod
    def on_fail(resp, message, status=falcon.HTTP_404):
        errmsg = OrderedDict()
        errmsg['404'] = f'{message}'
        resp.body = errmsg
        resp.status = status
        resp.reason = json.dumps(errmsg, indent=4, separators=(',', ': ')).encode('utf8')
        resp.content_type = 'application/json'

    @staticmethod
    def parse_return_code(resp, return_code):
        if return_code == 3:
            resp.status = falcon.HTTP_403
            Applications.on_fail(resp, 'Native applications not enabled in configuration')
        elif return_code == 4:
            resp.status = falcon.HTTP_403
            Applications.on_fail(resp, 'No valid license file')
        elif return_code == 5:
            resp.status = falcon.HTTP_403
            Applications.on_fail(resp, 'Native applications not allowed by a valid license')
        elif return_code == 6:
            resp.status = falcon.HTTP_403
            Applications.on_fail(resp, 'Missing application.json')
        elif return_code == 7:
            resp.status = falcon.HTTP_403
            Applications.on_fail(resp, 'Missing config.json')
        elif return_code == 8:
            resp.status = falcon.HTTP_403
            Applications.on_fail(resp, 'Operation not allowed in config.json')
        elif return_code == 9:
            resp.status = falcon.HTTP_403
            Applications.on_fail(resp, 'Operation is not rest-call-supported')
        elif return_code == 1:
            resp.status = falcon.HTTP_403
            Applications.on_fail(resp, 'Missing operation executable script')
        elif return_code == 11:
            resp.status = falcon.HTTP_403
            Applications.on_fail(resp, 'Application is not active')
        elif return_code == 12:
            resp.status = falcon.HTTP_403
            Applications.on_fail(resp, 'Application is active')
        elif return_code != 0:
            Applications.on_fail(resp, 'Operation fail')

    class Onboard(res.BaseResource):
        @falcon.before(send_notification.request_notifier)
        def on_put(self, req, resp):
            json_req = self.json_body(req)
            if 'application' not in json_req:
                Applications.on_fail(resp, 'No application')
                return

            appname = json_req['application']
            if not os.path.exists(f'/home/admin/user_files/{appname}.tar.gz'):
                Applications.on_fail(resp, f'No {appname}.tar.gz')
                return

            ret = subprocess.call(
                f'/usr/bin/sudo /scripts/application_onboard.sh {appname}',
                shell=True)
            if ret != 0:
                Applications.parse_return_code(resp, ret)
            else:
                resp.body = {'onboard': 'ok'}
                resp.status = falcon.HTTP_200

            return

    class Remove(res.BaseResource):
        @falcon.before(send_notification.request_notifier)
        def on_put(self, req, resp):
            json_req = self.json_body(req)
            if 'application' not in json_req:
                Applications.on_fail(resp, 'No application')
                return

            appname = json_req['application']
            if 'clavister' in appname.lower() and 'clavistervnf' not in appname.lower():
                subprocess.call(['/usr/bin/sudo', 'touch', '/run/.reboot_in_progress'])

            subprocess.call(f'/usr/bin/sudo systemctl disable {appname.lower()} || true', shell=True)
            subprocess.call(f'/usr/bin/sudo systemctl --no-block stop {appname.lower()}.service || true', shell=True)
            subprocess.call(f'/usr/bin/sudo rm -rf /root/apps/{appname.lower()} || true', shell=True)
            subprocess.call(f'/usr/bin/sudo systemctl reset-failed {appname.lower()} || true', shell=True)
            subprocess.call(f'/usr/bin/sudo rm -f /lib/systemd/system/{appname.lower()} || true', shell=True)
            subprocess.call('/usr/bin/sudo systemctl daemon-reload || true', shell=True)

            resp.body = {'remove': 'ok'}
            resp.status = falcon.HTTP_200
            if 'clavister' in appname.lower() and 'clavistervnf' not in appname.lower():
                if os.path.exists('/config/_reboot/tve/now'):
                    subprocess.call(['/usr/bin/sudo', 'rmdir', '/config/_reboot/tve/now'])

                subprocess.call(['/usr/bin/sudo', 'mkdir', '/config/_reboot/tve/now'])

            return

    class Start(res.BaseResource):
        @falcon.before(send_notification.request_notifier)
        def on_put(self, req, resp):
            json_req = self.json_body(req)
            if 'application' not in json_req:
                Applications.on_fail(resp, 'No application')
                return

            appname = json_req['application']

            ret = subprocess.call(
                f'/usr/bin/sudo /scripts/application_start.sh {appname.lower()}',
                shell=True)
            if ret != 0:
                Applications.parse_return_code(resp, ret)
            else:
                resp.body = {'start': 'ok'}
                resp.status = falcon.HTTP_200

            return

    class Stop(res.BaseResource):
        @falcon.before(send_notification.request_notifier)
        def on_put(self, req, resp):
            json_req = self.json_body(req)
            if 'application' not in json_req:
                Applications.on_fail(resp, 'No application')
                return

            appname = json_req['application']

            ret = subprocess.call(
                f'/usr/bin/sudo /scripts/application_stop.sh {appname.lower()}',
                shell=True)
            if ret != 0:
                Applications.parse_return_code(resp, ret)
            else:
                resp.body = {'stop': 'ok'}
                resp.status = falcon.HTTP_200

            return

    class List(res.BaseResource):
        @falcon.before(send_notification.request_notifier)
        def on_put(self, req, resp):
            # listdir - No Permissions
            auuid = uuid.uuid4()
            if not req.content_length:
                try:
                    ret = subprocess.call(
                        f'/usr/bin/sudo /scripts/application_list.sh all {auuid}',
                        shell=True)
                    if ret != 0:
                        Applications.parse_return_code(resp, ret)
                        return

                except Exception:
                    LOG.error('List fail!')
                    Applications.on_fail(resp, 'List fail!')
                    return

                try:
                    with open(f'/tmp/application_list_{auuid}.json', 'r') as json_file:
                        data = json.load(json_file)

                except Exception:
                    LOG.error('Reading application_list.json failed !')
                    Applications.on_fail(resp, 'List fail')
                    return

                resp.body = data
                resp.status = falcon.HTTP_200
                return

            json_req = self.json_body(req)
            if 'application' not in json_req:
                Applications.on_fail(resp, 'No application')
                return

            appname = json_req['application']
            ret = subprocess.call(
                f'/usr/bin/sudo /scripts/application_list.sh {appname.lower()} {auuid}',
                shell=True)
            if ret != 0:
                Applications.parse_return_code(resp, ret)
                return

            try:
                with open(f'/tmp/application_list_{auuid}.json', 'r') as json_file:
                    data = json.load(json_file)

            except Exception:
                LOG.error('Reading application_list.json failed !')
                Applications.on_fail(resp, f'List {appname.lower()} fail')
                return

            try:
                subprocess.call(
                    f'/usr/bin/sudo rm -f /tmp/application_list_{auuid}.json',
                    shell=True)
            except Exception:
                LOG.error('List fail!')
                Applications.on_fail(resp, 'List fail!')
                return

            resp.body = data
            resp.status = falcon.HTTP_200
            return

    class Retrieve(res.BaseResource):
        @falcon.before(send_notification.request_notifier)
        def on_put(self, req, resp):
            json_req = self.json_body(req)
            if 'application' not in json_req:
                Applications.on_fail(resp, 'No application')
                return

            appname = json_req['application']
            ret = subprocess.call(
                f'/usr/bin/sudo ls /root/apps/{appname.lower()}',
                shell=True)
            if ret != 0:
                Applications.on_fail(resp, f'Missing {appname.lower()}')
                return

            ret = subprocess.call(
                f'/usr/bin/sudo /scripts/application_retrieve.sh {appname.lower()}',
                shell=True)
            if ret != 0:
                Applications.parse_return_code(resp, ret)
            else:
                resp.body = {'retrieve': 'ok'}
                resp.status = falcon.HTTP_200

            return

    class RetrieveConfigs(res.BaseResource):
        # @falcon.before(send_notification.request_notifier)
        def on_put(self, req, resp):
            json_req = self.json_body(req)
            if 'application' not in json_req:
                Applications.on_fail(resp, 'No application')
                return

            appname = json_req['application']
            ret = subprocess.call(
                f'/usr/bin/sudo ls /root/apps/{appname.lower()}',
                shell=True)
            if ret != 0:
                Applications.on_fail(resp, 'Missing {appname.lower()}')
                return

            ret = subprocess.call(
                f'/usr/bin/sudo /scripts/application_retrieveconfigs.sh {appname.lower()}',
                shell=True)
            if ret != 0:
                Applications.parse_return_code(resp, ret)
            else:
                resp.body = {'retrieveconfigs': 'ok'}
                resp.status = falcon.HTTP_200

            return

    class Execute(res.BaseResource):
        @falcon.before(send_notification.request_notifier)
        def on_put(self, req, resp):
            json_req = self.json_body(req)
            if 'application' not in json_req:
                Applications.on_fail(resp, 'No application')
                return

            if 'operation' not in json_req:
                Applications.on_fail(resp, 'No operation')
                return

            auuid = uuid.uuid4()
            appname = json_req['application']
            operation = json_req['operation']

            try:
                out = subprocess.run(['/usr/bin/sudo',
                                      '/scripts/application_execute.sh',
                                      f'{appname}', f'{operation}', f'{auuid}'])

            except subprocess.CalledProcessError:
                Applications.on_fail(resp, 'Fail exec application_execute.sh')
                return
            except OSError:
                Applications.on_fail(resp, 'Missing sudo')
                return

            if 0 != out.returncode:
                LOG.error(f'ret code is {out.returncode}')
                Applications.parse_return_code(resp, out.returncode)
            else:
                try:
                    with open(f'/tmp/application_execute_{auuid}.out', 'r') as ef:
                        data = ef.read()

                    subprocess.call(['/usr/bin/sudo', 'rm', '-f',
                                     f'/tmp/application_execute_{auuid}.out'])

                except FileNotFoundError:
                    Applications.on_fail(resp, f'File /tmp/application_execute_{auuid}.out not found')
                    return
                except OSError:
                    Applications.on_fail(resp, f'OS error on /tmp/application_execute_{auuid}.out')
                    return
                except IOException:
                    Applications.on_fail(resp, f'IO on rm /tmp/application_execute_{auuid}.out')
                    return

                resp.body = {'execute': f'{data}'}
                resp.status = falcon.HTTP_200

            return

    class LoadConfig(res.BaseResource):
        @falcon.before(send_notification.request_notifier)
        def on_put(self, req, resp):
            json_req = self.json_body(req)
            if 'application' not in json_req:
                Applications.on_fail(resp, 'No application')
                return

            if 'config' not in json_req:
                Applications.on_fail(resp, 'No config')
                return

            appname = json_req['application']
            archive = json_req['config']
            if 'tar.gz' not in archive:
                Applications.on_fail(resp, f'Must be tar.gz archive')
                return

            if not os.path.exists(f'/home/admin/user_files/{archive}'):
                Applications.on_fail(resp, f'No {archive}')
                return

            ret = subprocess.call([
                '/usr/bin/sudo',
                '/scripts/application_loadconfig.sh',
                f'{appname}', f'{archive}'])
            if ret != 0:
                Applications.parse_return_code(resp, ret)
            else:
                resp.body = {'loadconfig': 'ok'}
                resp.status = falcon.HTTP_200

            return

    class Reset(res.BaseResource):
        # @falcon.before(send_notification.request_notifier)
        def on_put(self, req, resp):
            json_req = self.json_body(req)
            if 'application' not in json_req:
                Applications.on_fail(resp, 'No application')
                return

            appname = json_req['application']
            ret = subprocess.call(
                f'/usr/bin/sudo ls /root/apps/{appname.lower()}',
                shell=True)
            if ret != 0:
                Applications.on_fail(resp, 'Missing {appname.lower()}')
                return

            ret = subprocess.call(
                f'/usr/bin/sudo /scripts/application_reset.sh {appname.lower()}',
                shell=True)
            if ret != 0:
                Applications.parse_return_code(resp, ret)
            else:
                resp.body = {'reset': 'ok'}
                resp.status = falcon.HTTP_200

            return
