#!/usr/bin/env python3
import json
from oslo_log import log
import falcon
import webob.dec
import webob.exc
from tveapi.token import decode_token
from tveapi.token import decode_token_user
import grp
import pwd

from tveapi import context

LOG = log.getLogger(__name__)
request_json_err_msg = 'This API only supports requests encoded as JSON.'
response_json_err_msg = 'This API only supports responses encoded as JSON.'

class JSONTranslator(object):
    def process_response(self, req, resp, resource, req_succeeded):
        if not hasattr(resp, 'body'):
            return
        if isinstance(resp.data, dict):
            resp.data = json.dumps(resp.data)

        if isinstance(resp.body, dict):
            resp.body = json.dumps(resp.body)
            
class RequireJSON(object):
    def process_request(self, req, resp):
        if not req.client_accepts_json:
            LOG.error(request_json_err_msg)
            raise falcon.HTTPNotAcceptable(
                request_json_err_msg,
                href='http://docs.examples.com/api/json')


class Middleware(object):
    """
    WSGI wrapper for all tveapi middlewares. Use this will allow to manage
    middlewares through paste
    """

    def __init__(self, app):
        self.app = app

    def process_request(self, req):
        """
        implement this function in your middleware to change the request
        if the function return None the request will be handled in the next
        level functions
        """
        return None

    def process_response(self, response):
        """
        Implement this to modify your response
        """
        return response

    @classmethod
    def factory(cls, global_conf, **local_conf):
        def filter(app):
            return cls(app)

        return filter

    @webob.dec.wsgify
    def __call__(self, req, params=None):
        response = self.process_request(req)
        if response:
            return response
        response = req.get_response(self.app)
        response.req = req
        try:
            return self.process_response(response)
        except webob.exc.HTTPException as e:
            LOG.error(e)
            return e

class BaseContextMiddleware(Middleware):
    def process_response(self, response):
        try:
            request_id = response.req.context.request_id
        except AttributeError:
            LOG.warning('Unable to retrieve request id from context')
        else:
            # For python 3 compatibility need to use bytes type
            prefix = b'req-' if isinstance(request_id, bytes) else 'req-'

            if not request_id.startswith(prefix):
                request_id = prefix + request_id

            response.headers['x-openstack-request-id'] = request_id
        return response


class ContextMiddleware(BaseContextMiddleware):
    def get_context(self, req):
        token = req.headers.get('X-Auth-Token')
        userid = None
        domainid = req.headers.get('X-Domain-Id')
        tenantid = req.headers.get('X-Tenant-Id')
        user_domain_id = req.headers.get('X-User-Domain-Id')
        projectid = req.headers.get('X-Project-Id')
        project_domain_id = req.headers.get('X-Project-Domain-Id')
        request_id = req.headers.get('X-Openstack-Request-ID')
        roles_header = req.headers.get('X-Roles', '')
        roles = [r.strip().lower() for r in roles_header.split(',')]        

        try:
            userid = decode_token_user(token)
        except Exception as e:
            print('Failed getting user from token: {}'.format(e))

        try:
            groups = [g.gr_name for g in grp.getgrall() if userid in g.gr_mem]
            gid = pwd.getpwnam(userid).pw_gid
            groups.append(grp.getgrgid(gid).gr_name)
        except Exception as e:
            print('Failed setting user group: {}'.format(e))

        is_admin = userid in groups

        return context.TVEContext(auth_token=token,
                                      user=userid,
                                      tenant=tenantid or projectid,
                                      domain=domainid,
                                      user_domain=user_domain_id,
                                      project_domain=project_domain_id,
                                      is_admin=is_admin,
                                      request_id=request_id,
                                      resource_uuid=None,
                                      roles=groups
                                      )

    def process_request(self, req):
        req.context = self.get_context(req)
        req.environ['tveapi.context'] = req.context
        return None

        if req.headers.get('X-Identity-Status') == "Confirmed":
            # falcon is overwriting the context variable in the request
            # so we need to set the context in env variable
            req.context = self.get_context(req)
            req.environ['tveapi.context'] = req.context
        else:
            raise webob.exc.HTTPUnauthorized()
     
class CustomAuthMiddleware(object):
    def process_request(self, req, resp):
        #resp.complete = True
        LOG.info("Performing process_request for CustomAuthMiddleware")
        token = req.get_header('X-Auth-Token')
        LOG.info("CustomAuthMiddleware token: {}".format(token))
        
        if token is None:
            raise webob.exc.HTTPUnauthorized()
            
        if decode_token(token) != True:
            raise webob.exc.HTTPUnauthorized()
        

    def process_resource(self, req, resp, resource, params):
        #resp.complete = True
        LOG.info("Performing process_resource for CustomAuthMiddleware")
     
    def process_response(self, req, resp, resource, req_succeeded):
        #resp.complete = True
        LOG.info("Performing process_response for CustomAuthMiddleware")
