#!/usr/bin/env python3
import falcon
import json
import os
from oslo_log import log
from datetime import datetime
from collections import OrderedDict

from tveapi.common.utils import execute_shell_command

LOG = log.getLogger(__name__)

UCPE_CONFIG_FILE = '/config/ucpe_config.json'
HOMECALLING = '/scripts/homecalling-api.sh'
SYSTEM_BUSY_MSG = 'SYSTEM BUSY'
STORAGE_PATH = '/home/admin/user_files/'
UCPE_STATUS_FILE = '/config/_hb/ucpe_status.json'
ZTP_AUTH_KEY = '/config/_hb/ztp_auth.key'
BACKUP_FILE='/tmp/mysql_backup.sql'
LICENSE_STORAGE='/config/license'
TEMPORARY_DIR='/tmp/'
K8S_DHCP_DIR='/kubernetes/dhcp/'
CENTRAL_CONTROLLER = '/config/central-box'
K8S_SINGLE_MACHINE = '/config/k8s-single-machine'

class BaseResource(object):
    __tve_boot_dict = {
        "json": {
            "tve": {
                "boot":{}
            }
        }
    }

    __tve_config_dict = {
        "json": {
            "tve": {
                "config":{}
            }
        }
    }

    __tve_config_networking = {
        "json": {
            "tve": {
                "config":{
                    "networking":{}
                }
            }
        }
    }

    __tve_json_dict = {
        "json": {
            "tve": {}
        }
    }

    def get_tve_config_dict(self):
        return self.__tve_config_dict.copy()

    def get_networking_dict(self):
        return self.__tve_config_networking.copy()

    def get_tve_json_dict(self):
        return self.__tve_json_dict.copy()

    def get_tve_boot_dict(self):
        return self.__tve_boot_dict.copy()

    def execute_command(self, resp, command):
        exit_code, stdout, stderr = execute_shell_command(command)
        self.set_response(resp, stderr, stdout, exit_code)

    def execute_commands(self, resp, commands):
        outputs = {}
        for key in commands:
            exit_code, stdout, stderr = execute_shell_command(commands[key])
            if exit_code != 0:
                self.set_response(resp, stderr, stdout, exit_code)
            else:
                outputs.update({key:stdout})

        self.set_response(resp, stderr, outputs, exit_code)

    def call_homecalling(self, resp, full_doc, prefix_name):
        file_name = self.create_temp_file('{}_'.format(prefix_name), full_doc)
        code, stdout, stderr = execute_shell_command('sudo {} {} {}'.format(HOMECALLING, UCPE_CONFIG_FILE, file_name))
        if code != 0:
            self.set_status_500(resp, stderr, stdout)
        elif SYSTEM_BUSY_MSG in str(stdout):
            self.set_status_system_busy(resp)

    @staticmethod
    def max_body(limit = 64 * 1024):
        def hook(req, resp, resource, params):
            length = req.content_length
            if length is not None and length > limit:
                msg = ('The size of the request is too large. The body must not '
                       'exceed ' + str(limit) + ' bytes in length.')
                raise falcon.HTTPRequestEntityTooLarge(
                    'Request body is too large', msg)
        return hook

    @staticmethod
    def file_size(limit = 64 * 1024):
        def hook(req, resp, resource, params):
            length = req.content_length
            if length is not None and length > limit:
                msg = ('The size of file is too large. The file must not '
                       'exceed ' + str(limit/1024 ** 2) + 'MB.')
                raise falcon.HTTPPayloadTooLarge(
                    'Request body is too large.', msg)
        return hook

    @staticmethod
    def json_body(req):
        if not req.content_length:
            raise falcon.HTTPBadRequest('Missing request body')

        try:
            raw_json = req.stream.read()
        except Exception:
            raise falcon.HTTPBadRequest(
                'TTL Out of Range',
                'The messages TTL must be between 60 and 300 seconds, inclusive.'
            )
        try:
            json_data = json.loads(raw_json)
        except ValueError:
            raise falcon.HTTPBadRequest('Malformed JSON: ' + raw_json)
        return json_data

    def create_temp_file(self, file_name, doc):
        date = datetime.utcnow().strftime('%Y-%m-%d_%H_%M_%S_%f')[:-3]
        file_name = '{}{}.json'.format(file_name, date)

        with open(file_name, 'w+') as content_file:
            json_contents = json.dumps(doc,  sort_keys=True,
                                       indent=4, separators=(',', ': '))
            content_file.write(json_contents)
            LOG.info('File with name {} created: \n {}'
                     .format(file_name, json_contents))
            return file_name

    def set_status_500(self, resp, stderr="", stdout=""):
        resp.status = falcon.HTTP_INTERNAL_SERVER_ERROR
        resp.body = {'err_msg':'{}\n{}'.format(stderr, stdout)}

    def set_status_400(self, resp, stderr="", stdout=""):
        resp.status = falcon.HTTP_BAD_REQUEST
        resp.body = {'err_msg':'{}\n{}'.format(stderr, stdout)}

    def set_status_system_busy(self, resp):
        resp.status = falcon.HTTP_SERVICE_UNAVAILABLE
        resp.body = {'System temporary unavailable - ' : SYSTEM_BUSY_MSG}

    def set_response(self, resp, stderr, stdout, code = 0):
        if code != 0:
            self.set_status_500(resp, stderr, stdout)
        else:
            resp.body = stdout

    def on_fail(self, resp, message):
        errmsg = OrderedDict()
        errmsg['404'] = ("{}".format(message))
        resp.body = errmsg
        resp.status = falcon.HTTP_404
        resp.data = json.dumps(
            errmsg, indent=4, separators=(',', ': ')).encode('utf8')
        resp.content_type = 'application/json'

    @staticmethod
    def is_k8s_controller():
        return os.path.isfile(CENTRAL_CONTROLLER)
    @staticmethod
    def is_k8s_single_machine():
        return os.path.isfile(K8S_SINGLE_MACHINE)

    @staticmethod
    def k8s_control_plane_eligible():
        def hook(req, resp, resource, params):
            if not any([BaseResource.is_k8s_controller(), BaseResource.is_k8s_single_machine()]):
                raise falcon.HTTPBadRequest('Cannot execute the request on worker node')
        return hook
