#!/usr/bin/env python3
import json
from six import string_types
#import syslog

class JsonPathException(Exception):
    pass

def get_json_dict_from_file(file):
    """
    Returns a dictionary object, constructed from the JSON String in file
    :file - A file containing valid JSON
    """
    with open(file, 'r') as content_file:
        json_content = content_file.read().rstrip()
        return json.loads(json_content)

def write_json_file(json_dict, target_file):
    """
    Writes a dictionary to a file.
    :json_dict - The dictionary to be dumped to the file
    :target_file - The file to be written
    """
    with open(target_file, 'w') as content_file:
        json_contents = json.dumps(json_dict,  sort_keys=True,
                 indent=4, separators=(',', ': '))
        content_file.write(json_contents)

def get_node(tree, path):
    """
    Returns dictionary extracted from
    :param tree
    :param path - The JSON path to search
    """

    def dfs(tree, search):
        if type(tree) is not dict:
            return None
        for key in tree.keys():
            if key == search:
                return tree[key]
            else:
                ret = dfs(tree[key],search)
                if ret is not None:
                    return ret

    path = path.strip('/')
    if not path:
        return tree

    keys = path.split("/")
    while keys:
        tree = dfs(tree, keys[0])
        keys = keys[1:]
    if tree is None:
        return dict()
    return tree

def sync_json_sections(path, source_file, target_file):
    """
    Synchronizes the json sections from  source_file to target_file
    :path - The JSON path specifying the sections to be synchronized
    :source_file - The file to synchronizes from
    :target_file -  The file to synchronizes to
    """
    if  path.startswith("//"):
        full_path = False
    else:
        full_path = True

    path = path.strip('/')
    src = get_json_dict_from_file(source_file)

    if not path:
        trg = src
    else:
        if full_path:
            trg = get_json_dict_from_file(target_file)

            #get the part that is to be updated in the target
            res_src = get_node(src, path)

            path_list =  path.split('/')
            key = path_list[-1]
            new_path = "".join(str(i) + "/" for i in path_list[0 : len(path_list) -1])

            #add the missing keys in the target dict's json
            target = trg
            for node in path_list:
                if node not in target or type(target[node]) is not dict:
                    target.update({node:{}})
                target = target[node]

            #update the target dict
            res_trg = get_node(trg, new_path)
            if type(res_trg) is  dict:
                res_trg[key]=res_src
            else:
                res_trg = {key:res_src}

            write_json_file(trg, target_file)
        # we are in the relative path
        else:
            res_src = get_node(src, path)


            full_path = get_full_path(src, path)
            sync_json_sections (full_path, source_file, target_file)

def _get_json_value(path, json_file):
    path = path.strip('/')
    json_dict = get_json_dict_from_file(json_file)
    json_section = get_node(json_dict, path)
    if isinstance(json_section, dict) and not json_section:
        return ""
    return json_section

def get_json_value(path, json_file):
    """
    Prints the JSON section specified
    :path The JSON path to the section to be printed
    :json_file A file containing valid Json
    """
    json_section = _get_json_value(path, json_file)
    if isinstance(json_section, string_types):
        return str(json_section).strip('"')
    else:
#         return json_section
        return(json.dumps(json_section,  sort_keys=True,
                  indent=4, separators=(',', ': ')))

def compare_json_sections(path, file1, file2):
    """
    Compares two JSON sections. Returns True if
    the sections are equal, False otherwise.
    :path - the JSON path to the sections to be compared
    :file1 - the first file
    :file2 - the second file
    """
    path = path.strip('/')
    file1_dict = get_json_dict_from_file(file1)
    file2_dict = get_json_dict_from_file(file2)

    section1 = get_node(file1_dict, path)
    section2 = get_node(file2_dict, path)
    if ((type(section1) is dict and not bool(section1)) or
            type(section2) is dict and not bool(section2)):
        return 2
    return section1 == section2

def normalize_value(val):
    if val is None:
        return ""
    if isinstance(val, bool):
        return str(val).lower()
    else:
        return val

def _get_json_value_by_key(json_dict, path, query, element=None):
    path = path.strip('/')
    target_tree = get_node(json_dict, path)

    if type(target_tree) is dict:
        target_tree = [ v for v in target_tree.values() ]

    srch_key,srch_val = query.split("=")
    for obj in target_tree:
            if str(obj.get(srch_key)).lower() == srch_val.lower():
                return obj if element is None else \
                    normalize_value(obj.get(element))
    return ""

def get_json_value_by_key(file, path, key, element=None):
    """
    Returns the value of "element" in the json tree, specified by the given key.
    """
    json_dict = get_json_dict_from_file(file)
    return _get_json_value_by_key(json_dict, path, key, element)

def get_json_values_by_key(json_file, path, key, element=None):
    """
    Prints the JSON section specified
    :path The JSON path to the section to be printed
    :json_file A file containing valid Json
    """
    srch_key,srch_val = key.split("=")
    json_section = _get_json_value(path, json_file)
    json_filters = [x for x in json_section if x.get(srch_key) == srch_val]
    if isinstance(json_filters, string_types):
        return str(json_filters).strip('"')
    elif element:
        return " ".join([x.get(element) for x in json_filters])
    return(json.dumps(json_filters,  sort_keys=True,
                      indent=4, separators=(',', ': ')))

def list_to_path_string(full_path_list):
    return  "/".join(str(i) for i in
                    full_path_list[0 : len(full_path_list)])

def get_full_path(tree, path):

    def dfs(tree, search):
        if type(tree) is not dict:
            return None
        for key in tree.keys():
            full_path_list.append(key)
            if key == search:
                return tree[key]
            else:
                ret = dfs(tree[key],search)
                if ret is not None:
                    return ret
                else:
                    pass
                    full_path_list.pop()

    full_path_list = list()

    keys = path.split("/")
    while keys:
        tree = dfs(tree, keys[0])
        keys = keys[1:]

    full_path = list_to_path_string(full_path_list)

    return full_path

def get_nested_node( my_dict, keys=[] ):
    if len(keys) == 0:
        return my_dict

    key = keys.pop(0)

    return get_nested_node(my_dict[key], keys)

def sync_cfg_section_json( src, dst, keys=[] ):
    dict_dst = get_nested_node( dst, list( keys ) )
    dict_src = get_nested_node( src, list( keys ) )

    if 'dict' == type( dict_dst ).__name__:
        dict_dst.update( dict_src )
    else:
        #last value is not dictionary - it is value
        dict_dst = get_nested_node( dst, list( keys[:-1] ) )
        dict_src = get_nested_node( src, list( keys[:-1] ) )
        dict_dst[ keys[-1] ] = dict_src[ keys[-1] ]

def sync_cfg_section( src_file, dst_file, keys=[] ):
    json_src = open( src_file, "r+" )
    json_dst = open( dst_file, "r+" )

    src_cfg = json.load( json_src )
    dst_cfg = json.load( json_dst )

    sync_cfg_section_json( src_cfg, dst_cfg, keys )
    # dict_dst = get_nested_node( dst_cfg, list( keys ) )
    # dict_src = get_nested_node( src_cfg, list( keys ) )

    # if 'dict' == type( dict_dst ).__name__:
    #     dict_dst.update( dict_src )
    # else:
    #     #last value is not dictionary - it is value
    #     dict_dst = get_nested_node( dst_cfg, list( keys[:-1] ) )
    #     dict_src = get_nested_node( src_cfg, list( keys[:-1] ) )
    #     dict_dst[ keys[-1] ] = dict_src[ keys[-1] ]

    #syslog.syslog(syslog.LOG_DEBUG, json.dumps( dst_cfg ) )
    #syslog.syslog(syslog.LOG_DEBUG, str(dict_dst) )
    #syslog.syslog(syslog.LOG_DEBUG, str(dict_src) )

    json_dst.seek(0)
    json_dst.truncate()
    json_dst.seek(0)

    json.dump( dst_cfg, json_dst, sort_keys=True, indent=4, separators=(',', ': ') )
