#!/usr/bin/env python3
import json
import jwt
import falcon
from oslo_log import log
#from datetime import datetime, timedelta, timezone
from datetime import datetime, timedelta
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.backends import default_backend
from tveapi.common import resource as res
from tveapi.common import utils
import warnings
warnings.filterwarnings("ignore")

LOG = log.getLogger(__name__)
private_key_file = "/config/etc/keys/private.pem"
public_key_file = "/config/etc/keys/public.pem"
PASSW_EXP_PERIOD_MIN = 60*24

def _xpath_dot_split(key):
    """
    Internal function used for get_value
    """
    items = []
    parts = str(key).split('.')
    i = 0
    while i < len(parts):
        if parts[i].find('[') > -1 and parts[i][parts[i].find('['):].find(']') == -1:
            data = parts[i]
            while i < len(parts):
                i += 1
                data += '.' + parts[i]
                if parts[i].find(']') > -1:
                    items.append(data)
                    i += 1
                    break
            continue
        items.append(parts[i])
        i += 1
    return items


def get_value(json_config, key, default=None):
    """
    Get a value from json config in dot separated format.
    Note that this search works with indexes if the json contains lists.
    For example: ucpe.config.interfaces['name'='data0'].metric
    This will search in the interfaces list for item with parameter name matching 'data0'
    and will continue with its elements.
    :param json_config: Json configuration
    :param key: The key to be found in the Json.
    :param default: The default value to be returned in case the key was not found.
    :return: Value of the key.
    """
    if not isinstance(json_config, dict):
        raise Exception("Invalid json format! {}".format(json_config))
    d = json_config
    keys = _xpath_dot_split(key)
    for d_key in keys[:-1]:
        if d_key.find('[') > -1 and d_key.find('[') > -1:
            # list index, find the key
            index = d_key[:d_key.find('[')]
            index_pair = d_key[d_key.find('[')+1:d_key.find(']')]
            index_key = index_pair[:index_pair.find('=')]
            index_value = index_pair[index_pair.find('=')+1:].replace('\'', '').replace('"', '')
            if index not in d:
                return default
            i = 0
            is_found = False
            for list_value in d[index]:
                if index_key in list_value and str(list_value[index_key]) == index_value:
                    d = d[index][i]
                    is_found = True
                    break
                i += 1
            if not is_found:
                return default
            continue
        if d_key in d:
            d = d[d_key]
            continue
        return default
    if (isinstance(d, dict) or isinstance(d, list)) and keys[-1] in d:
        return d[keys[-1]]
    return default


def set_value(json_config, key, value, create_missing=True, raise_exception=True):
    """
    Set a value in complex json config using dot separated keys. For more information look ata get_value.
    :param json_config: Json to be used as basis.
    :param key: Key to be set.
    :param value: Value to be set.
    :param create_missing: By default create the missing json structure in order to srt the value
    :param raise_exception: Raise exception by default.
    :return: Return the updated json with set value.
    """
    str_data = "locals()['json_config']"
    keys = _xpath_dot_split(key)
    for key in keys[:-1]:
        if key.find('[') > -1 and key.find(']') > -1 and key.find('=') > -1:
            # list index, find the key
            index = key[:key.find('[')]
            index_pair = key[key.find('[')+1:key.find(']')]
            index_key = index_pair[:index_pair.find('=')]
            index_value = index_pair[index_pair.find('=')+1:].replace('\'', '').replace('"', '')
            i = 0
            is_found = False
            if index in eval(str_data):
                for list_value in eval("{}['{}']".format(str_data, index)):
                    if isinstance(list_value, dict) and index_key in list_value and str(list_value[index_key]) == index_value:
                        is_found = True
                        str_data = "%s['%s'][%d]" % (str_data, index, i)
                        break
                    if list_value == index_key and eval("{}['{}']['{}']".format(str_data, index, index_key)) == index_value:
                        is_found = True
                        str_data = "%s['%s']" % (str_data, index)
                        break
                    i += 1
            if not is_found:
                if not create_missing:
                    if raise_exception:
                        raise Exception('Missing element {}'.format(key))
                else:
                    # add to list
                    if index not in eval(str_data):
                        exec("%s['%s'] = []" % (str_data, index))
                    exec("%s['%s'].append({index_key: index_value})" % (str_data, index))
                    str_data = "%s['%s'][%d]" % (str_data, index, len(eval("{}['{}']".format(str_data, index)))-1)
            continue
        if key in eval(str_data):
            str_data = "%s['%s']" % (str_data, key)
            continue
        if not create_missing:
            if raise_exception:
                raise Exception('Missing element {}'.format(key))
        else:
            exec("%s['%s'] = {}" % (str_data, key))
            str_data = "{}['{}']".format(str_data, key)
    if keys[-1] in eval(str_data):
        if create_missing:
            exec("{}['{}'] = value".format(str_data, keys[-1]))
    else:
        if not create_missing:
            if raise_exception:
                raise Exception('Missing element {}'.format(keys[-1]))
        else:
            exec("{}['{}'] = value".format(str_data, keys[-1]))
    return json_config


def get_token(user, passw, group, expire_mins = PASSW_EXP_PERIOD_MIN):
    with open(private_key_file, "rb") as f:
        private_key = serialization.load_pem_private_key(
            f.read(), None, backend=default_backend()
        )
    issd = datetime.utcnow()
    exp = issd + timedelta(minutes=expire_mins)
    json_data = {"nbf": issd, "user": user, "passw": passw, "grp": group, "exp": exp}
    token = jwt.encode(json_data, private_key, algorithm="RS256")
    LOG.info("Issuing token: {}". format((json_data)))
    return token
    

def decode_token(token):
    with open(public_key_file, "rb") as f:
        public_key = serialization.load_pem_public_key(
            f.read(), backend=default_backend()
        )
            
    decoded = jwt.decode(token, key=public_key, algorithms=["RS256"])
                
    issued = get_value(decoded, 'nbf', '')
    user = get_value(decoded, 'user', '')
    passw = get_value(decoded, 'passw', '')
    exp = get_value(decoded, 'exp', '')
    #LOG.error("{} {} {} {}".format(issued, user, passw, exp))
    LOG.info("Received token: {}". format(json.dumps(decoded)))
    
    code, stdout, stderr = utils.execute_shell_command("sudo /scripts/authenticate.py {} {}".format(user, passw), True)
    LOG.info("authenticate.py: code {} stdout {}".format(code, stdout))
    
    if code == 0:
        return True
        
    return False


def decode_token_user(token):
    with open(public_key_file, "rb") as f:
        public_key = serialization.load_pem_public_key(
            f.read(), backend=default_backend()
        )

    decoded = jwt.decode(token, key=public_key, algorithms=["RS256"])

    # we do not need to authenticate token again, we just need to get the user from already authenticated token 
    user = get_value(decoded, 'user', '')
    if user:
        return user
    else:
        return None


def check_token_expired(token):
    with open(public_key_file, "rb") as f:
        public_key = serialization.load_pem_public_key(
            f.read(), backend=default_backend()
        )                
    try:
        decoded = jwt.decode(token, key=public_key, algorithms=["RS256"])
        LOG.error("Token is valid!")
        return False
    except jwt.ExpiredSignatureError:
        # Signature has expired
        LOG.error("Token has expired!")
        return True


#def execute_shell_command(cmd, shell=True, ignore_error=False):
#    if cmd is str:
#        cmd = str(cmd).split(' ')
#    process = subprocess.Popen(
#        cmd,
#        stdout=subprocess.PIPE,
#        stderr=subprocess.PIPE,
#        shell=shell
#    )
#    (stdout, stderr) = process.communicate()
#    if len(stderr) > 0 and process.returncode > 0 and not ignore_error:
#        raise Exception("Error executing command {}!\n{}".format(cmd, stderr))
#    result = stdout.decode("utf-8")
#    return result

#Use the internal system user. See local-openrc.env file
#curl -k --insecure -d '{"user":"OS_USERNAME","password":"OS_PASSWORD"}' -H "Content-type: application/json" -X GET https://127.0.0.1:17374/v3/auth/tokens

class auth_token(res.BaseResource):
    def on_get(self, req, resp):
        LOG.info("token GET method called")
        json_req = self.json_body(req)
        name = json_req['user']
        password = json_req['password']
        LOG.info("token ON GET user {} pass {}".format(name, password))
        
        try:
            code, stdout, stderr = utils.execute_shell_command("sudo /scripts/authenticate.py {} {}".format(name, password), True)
            if code != 0:
                resp.status = falcon.HTTP_400
                #resp.status = falcon.HTTP_INTERNAL_SERVER_ERROR
                #resp.body = {'err_msg':'{}\n{}'.format(stderr, stdout)}
                return
            code, stdout, stderr = utils.execute_shell_command("groups {}".format(name), True)
            group = str(stdout).split(' : ')[1].strip()
            token = get_token(user=name, passw=password, group=group, expire_mins=60*24)
            resp.body = "%s" % (token)
            resp.status = falcon.HTTP_200
            return
        except Exception as ex:
            resp.status = falcon.HTTP_400
            raise Exception('Cannot get token!')
            
    def on_put(self, req, resp):
        LOG.info("token PUT method called")
        resp.status = falcon.HTTP_400
        return
