#!/usr/bin/env python3
import syslog
import json
import subprocess
import falcon
import xml.etree.ElementTree as ElT


def get_gpu_info_json():
    try:
        # Get the GPU xml file
        nvidia_smi_output = subprocess.check_output(["nvidia-smi", "-q", "-x"]).decode("utf-8")
        root = ElT.fromstring(nvidia_smi_output)
    except subprocess.CalledProcessError as e:
        syslog.syslog(syslog.LOG_ERR, f"Error running nvidia-smi: {e}")
        return []  # Return an empty list if the nvidia-smi command fails
    except ElT.ParseError as e:
        syslog.syslog(syslog.LOG_ERR, f"Error parsing XML from nvidia-smi: {e}")
        return []  # Return an empty list if there's a parsing error
    except FileNotFoundError as e:
        syslog.syslog(syslog.LOG_ERR, f"Error: {e}. Nvidia-smi command not found.")
        return []  # Return an empty list if nvidia-smi is not found

    device_spec = [
        {1: 'product_name'},
        {2: 'driver_version'},
        {3: 'vbios_version'},
        {4: 'current_link_width'}
    ]

    try:
        gpu_ids = [gpu.get("id") for gpu in root.findall(".//gpu")]
    except AttributeError as e:
        syslog.syslog(syslog.LOG_ERR, f"Error getting GPU IDs: {e}")
        return []  # Return an empty list in case of error

    # Check if there are no GPUs
    if not gpu_ids:
        return []

    gpu_info_list = []

    for gpu_id in gpu_ids:
        gpu_info = {'gpu_id': gpu_id}

        for param_dict in device_spec:
            for param_num, param_name in param_dict.items():
                try:
                    element = root.find(f".//gpu[@id='{gpu_id}']/{param_name}")
                    value = element.text.strip() if element is not None else "N/A"
                    gpu_info[param_name.lower()] = value
                except AttributeError as e:
                    # Handle the case when the parameter is missing
                    syslog.syslog(syslog.LOG_ERR, f"Error getting parameter '{param_name}' for GPU ID {gpu_id}: {e}")

        try:
            # Check for driver_version outside the loop
            driver_version_element = root.find(f".//gpu[@id='{gpu_id}']/../driver_version")
            current_link_width = root.find(f".//gpu[@id='{gpu_id}']/pci/pci_gpu_link_info/link_widths/current_link_width")
            gpu_info[
                'driver_version'] = driver_version_element.text.strip() if driver_version_element is not None else "N/A"
            gpu_info[
                'current_link_width'] = current_link_width.text.strip() if current_link_width is not None else "N/A"
        except AttributeError as e:
            # Handle the case when driver_version is missing
            syslog.syslog(syslog.LOG_ERR, f"Error getting 'driver_version' for GPU ID {gpu_id}: {e}")

        gpu_info_list.append(gpu_info)

    return gpu_info_list


class GpuInfoResource:

    def on_get(self, req, resp):
        gpu_info_list = get_gpu_info_json()
        resp.body = json.dumps(gpu_info_list)  # Convert list to JSON string
        resp.status = falcon.HTTP_200
        return resp.body
