import os
import subprocess
import falcon
import re
import syslog
import math
from tveapi.common import resource as res
from tveapi import policy
from tveapi.common import utils

MAX_UPLOAD_BYTE_LENGTH = 1024 * 1024


def convert_size(size_bytes):
    if size_bytes == 0:
        return "0B"
    size_name = ("B", "KB", "MB", "GB")
    i = int(math.floor(math.log(size_bytes, 1024)))
    p = math.pow(1024, i)
    s = round(size_bytes / p, 2)
    return "%s %s" % (s, size_name[i])


class LargerFile(res.BaseResource):

    TEMP_DIR = '/tmp'

    """
    This class is designed to handle the upload of large files.
    The code within it allows the Manager to upload files in smaller, manageable chunks.
    It relies on the "Range" header in the HTTP request to extract 'start', 'end', and 'total_bytes' values with regex.
    The 'start' value determines where each upload segment begins.
    The file that needs to be saved is opened in binary mode and the incoming data is written in portions,
    as the chucks are being appended until the file is completely sent.

    """
    @policy.enforce('larger-files:get')
    def on_get(self, req, resp):
        resp.status = falcon.HTTP_200
        resp.content_length = 12
        resp.content_type = 'application/octet-stream'
        if not req._params:
            try:
                command = "ls " + res.STORAGE_PATH
                available_files = subprocess.check_output(command, shell=True).decode()
                resp.body = str(available_files)
            except Exception as ex:
                self.set_status_400(resp, 'User folder does not exist')
                return
        else:
            if len(req._params) > 1:
                self.set_status_400(
                    resp, 'Only one search parameter is supported')
                return
            for key, value in req._params.items():
                filename = res.STORAGE_PATH + value
                try:
                    with open(filename, 'rb') as f:
                        resp.body = f.read()
                        f.close()
                    resp.downloadable_as = filename
                except Exception as ex:
                    self.set_status_400(resp, 'File does not exist')
                    syslog.syslog(syslog.LOG_ERR, f"Getting the file failed due to the following error {ex}")
                    return

    def on_post(self, req, resp):
        # Get the 'Range' header from the request, which specifies the byte range of the incoming data.
        range_header = req.get_header('Range')
        if range_header is None:
            syslog.syslog(syslog.LOG_ERR, "Missing 'Range' header")
            resp.text = "Missing 'Range' header"
            resp.status = falcon.HTTP_400  # Bad Request
            return

        # Parse the 'Range' header to extract start, end, and total_bytes information using regular expressions.

        match = re.search(r'(?P<start>\d+)-(?P<end>\d+)/(?P<total_bytes>\d+)', range_header)
        if not match:
            syslog.syslog(syslog.LOG_ERR, "Invalid 'Range' header format")
            resp.text = "Invalid 'Range' header format"
            resp.status = falcon.HTTP_400  # Bad Request
            return

        # Extract the 'start' value from the matched data.
        # This indicates the starting byte position for the incoming data chunk.

        start = int(match.group('start'))
        total_bytes = int(match.group('total_bytes'))
        file_size = convert_size(total_bytes)

        if not os.path.exists(res.STORAGE_PATH):
            utils.execute_shell_command('sudo mkdir {} -m 777'.format(res.STORAGE_PATH))

        # Add this line to log headers
        syslog.syslog(syslog.LOG_INFO, f"Received headers: {req.headers}")

        # Create a temporary directory for storing the file
        try:
            file_name = os.path.basename(req.get_header('Filename'))
        except Exception as e:
            syslog.syslog(syslog.LOG_ERR, f"The header is not found due to: {e}")
            return f"The error is {e}"

        temp_file_path = '/tmp/{}'.format(file_name)
        try:
            with open(temp_file_path, 'rb+' if os.path.exists(temp_file_path) else 'wb+') as f:
                f.seek(start)
                chunk = req.bounded_stream.read()
                f.write(chunk)
                current_file_size = convert_size(start)
                syslog.syslog(syslog.LOG_INFO, f" {current_file_size}/{file_size} from file "
                                               f"{file_name} was written to  {temp_file_path}.")
        except Exception as ex:
            # Handle any exceptions, e.g., clean up the temporary directory if an error occurs
            syslog.syslog(syslog.LOG_ERR, f"File {file_name} was not received due to {ex}")

            resp.text = str(ex)
            resp.status = falcon.HTTP_500  # Internal Server Error
            return

        # Check if the whole file has been received
        file_size = os.path.getsize(temp_file_path)

        if file_size >= total_bytes:
            # If the whole file has been received, move it to the final destination
            final_file_path = os.path.join(res.STORAGE_PATH, file_name)
            syslog.syslog(syslog.LOG_INFO, f"Move {temp_file_path} to {final_file_path}")
            os.rename(temp_file_path, final_file_path)

        resp.status = falcon.HTTP_200