import os

import falcon
from oslo_log import log
from tveapi import policy
from tveapi.common import resource as res
from tveapi.common import utils
from tveapi.common import validate_docs as docs
from tveapi.json_tools import json_tools
import sys
sys.path.insert(1, '/usr/lib/python3/dist-packages/tveapi/')
import send_notification

LOG = log.getLogger(__name__)


class Config(res.BaseResource):
    @policy.enforce('config:get')
    #@falcon.before(send_notification.request_notifier)
    def on_get(self, req, resp):
        code, stdout, stderr = utils.execute_shell_command('sudo /scripts/sysrepocfg.sh -X -x "/ucpe:config//*" -d running -f json')
        if code != 0:
            self.set_status_500(resp, stderr, stdout)
        elif res.SYSTEM_BUSY_MSG in str(stdout):
            self.set_status_system_busy(resp)
        else:
            resp.body = stdout

    @policy.enforce('config:put')
    @falcon.before(res.BaseResource.max_body())
    @falcon.before(send_notification.request_notifier)
    def on_put(self, req, resp):
        doc = self.json_body(req)
        workfile = self.create_temp_file('tveapi_import', doc)
        code, stdout, stderr = utils.execute_shell_command('sudo /scripts/sysrepocfg.sh --import={} -d running -f json -m ucpe'.format(workfile))
        if code != 0:
            self.set_status_500(resp, stderr, stdout)
        elif res.SYSTEM_BUSY_MSG in str(stdout):
            self.set_status_system_busy(resp)
        else:
            self.set_response(resp, stderr, stdout, code)

        os.remove(workfile)


class TVE(object):
    class ImageServer(res.BaseResource):
        @policy.enforce('config.image-server:get')
        def on_get(self, req, resp):
            resp.body = json_tools.get_json_value('//image-server', res.UCPE_CONFIG_FILE)

        @policy.enforce('config.image-server:put')
        @falcon.before(res.BaseResource.max_body())
        def on_put(self, req, resp):
            doc = self.json_body(req)
            docs.ServerDoc().validate(doc)

            full_doc = self.get_tve_boot_dict()
            full_doc['json']['tve']['boot'] = {'image-server': doc}

            self.call_homecalling(resp, full_doc, 'image-server')

    class ScriptsServer(res.BaseResource):
        @policy.enforce('config.script-server:get')
        def on_get(self, req, resp):
            resp.body = json_tools.get_json_value('//scripts-server', res.UCPE_CONFIG_FILE)

        @policy.enforce('config.script-server:put')
        @falcon.before(res.BaseResource.max_body())
        def on_put(self, req, resp):
            doc = self.json_body(req)
            docs.ServerDoc().validate(doc)

            full_doc = self.get_tve_boot_dict()
            full_doc['json']['tve']['boot'] = {'scripts-server': doc}

            self.call_homecalling(resp, full_doc, 'scripts-server')

    class ConfigServer(res.BaseResource):
        @policy.enforce('config.config-server:get')
        def on_get(self, req, resp):
            result = json_tools.get_json_value('//boot/config-server/ipv4-address',
                                               res.UCPE_CONFIG_FILE)
            resp.body = {"ipv4-address": result}

        @policy.enforce('config.config-server:put')
        @falcon.before(res.BaseResource.max_body())
        def on_put(self, req, resp):
            doc = self.json_body(req)
            docs.ConfigServerDoc().validate(doc)

            full_doc = self.get_tve_boot_dict()
            full_doc['json']['tve']['boot'] = {'config-server': doc}

            self.call_homecalling(resp, full_doc, 'config-server')

    class Homecalling(res.BaseResource):
        @policy.enforce('config.homecalling:get')
        def on_get(self, req, resp):
            resp.body = json_tools.get_json_value('//boot/homecalling', res.UCPE_CONFIG_FILE)

        @policy.enforce('config.homecalling:put')
        @falcon.before(res.BaseResource.max_body())
        def on_put(self, req, resp):
            doc = self.json_body(req)
            docs.HomecallingDoc().validate(doc)

            full_doc = self.get_tve_boot_dict()
            full_doc['json']['tve']['boot'] = {'homecalling': doc}

            self.call_homecalling(resp, full_doc, 'homecalling')

    class Networking(res.BaseResource):
        @policy.enforce('config.networking:get')
        def on_get(self, req, resp):
            resp.body = json_tools.get_json_value('//networking', res.UCPE_CONFIG_FILE)

        @policy.enforce('config.networking:put')
        @falcon.before(res.BaseResource.max_body())
        def on_put(self, req, resp):
            doc = self.json_body(req)
            docs.NetworkDoc().validate(doc)

            full_doc = self.get_tve_config_dict()
            full_doc['json']['tve']['config'] = {'networking': doc}

            self.call_homecalling(resp, full_doc, 'networking')

    class Hostname(res.BaseResource):
        @policy.enforce('config.hostname:get')
        def on_get(self, req, resp):
            result = json_tools.get_json_value('//hostname',
                                               res.UCPE_CONFIG_FILE)
            resp.body = {"hostname": result}

        @policy.enforce('config.hostname:put')
        @falcon.before(res.BaseResource.max_body())
        def on_put(self, req, resp):
            doc = self.json_body(req)
            docs.HostnameDoc.validate(doc)

            full_doc = self.get_tve_config_dict()
            full_doc['json']['tve']['config'] = doc

            self.call_homecalling(resp, full_doc, 'hostname')

    class SerialNumber(res.BaseResource):
        @policy.enforce('config.serial-number:get')
        def on_get(self, req, resp):
            self.execute_command(resp, 'sudo cat /config/.serial-number')

    class LCTConfig(res.BaseResource):
        @policy.enforce('config.lct:get')
        def on_get(self, req, resp):
            commands = {
                'serial_number': 'sudo cat /config/.serial-number',
                'version': 'dpkg -s nfvtimeos | grep -Po \"(?<=^Version: )([=><:+0-9~.a-zA-Z\-]*)$\"',
                'interfaces': 'sudo bash /scripts/cli/tvesh/show-port.sh physical csv | cut -d \',\' -f1,6',
                'hwtype': 'sudo cat /config/.hwtype',
                'status': 'grep -Po \'"status": "\K.*?(?=")\' /config/_hb/ucpe_status.json',
                'nsStatus': 'grep -Po \'"nsStatus": "\K.*?(?=")\' /config/_hb/ucpe_status.json',
                'ztpStatus': 'grep -Po \'"ztp-status": "\K.*?(?=")\' /config/_hb/ucpe_status.json'
            }
            self.execute_commands(resp, commands)

    class HWType(res.BaseResource):
        @policy.enforce('config.hardware-type:get')
        def on_get(self, req, resp):
            resp.body = json_tools.get_json_value('//hardware/type', res.UCPE_CONFIG_FILE)

        @policy.enforce('config.hardware-type:put')
        @falcon.before(res.BaseResource.max_body())
        def on_put(self, req, resp):
            doc = self.json_body(req)
            docs.HWTypeDoc.validate(doc)

            full_doc = self.get_tve_json_dict()
            full_doc['json']['tve'] = {'hardware': doc}

            self.call_homecalling(resp, full_doc, 'hardware')

    class NTP(res.BaseResource):
        @policy.enforce('config.ntp:get')
        def on_get(self, req, resp):
            resp.body = json_tools.get_json_value('//ntp', res.UCPE_CONFIG_FILE)

        @policy.enforce('config.ntp:put')
        @falcon.before(res.BaseResource.max_body())
        def on_put(self, req, resp):
            doc = self.json_body(req)
            docs.NTPDoc().validate(doc)

            full_doc = self.get_tve_config_dict()
            full_doc['json']['tve']['config'] = {'ntp': doc}

            self.call_homecalling(resp, full_doc, 'ntp')

    class RoutingStatic(res.BaseResource):
        @policy.enforce('config.static-routing:get')
        def on_get(self, req, resp):
            resp.body = json_tools.get_json_value('//routing/static', res.UCPE_CONFIG_FILE)

    class SecurityAuthentication(res.BaseResource):
        @policy.enforce('config.security.authentication:get')
        def on_get(self, req, resp):
            resp.body = json_tools.get_json_value('//security/authentication', res.UCPE_CONFIG_FILE)

        @policy.enforce('config.security.authentication:put')
        @falcon.before(res.BaseResource.max_body())
        def on_put(self, req, resp):
            doc = self.json_body(req)
            docs.SecurityAuthDoc().validate(doc)

            full_doc = self.get_tve_config_dict()
            full_doc['json']['tve']['config']['security'] = {'authentication': doc}

            self.call_homecalling(resp, full_doc, 'security_authentication')

    class SecurityCliWhitelist(res.BaseResource):
        @policy.enforce('config.security.cli-whitelist:get')
        def on_get(self, req, resp):
            resp.body = json_tools.get_json_value('//security/cli-whitelist', res.UCPE_CONFIG_FILE)

        @policy.enforce('config.security.cli-whitelist:put')
        @falcon.before(res.BaseResource.max_body())
        def on_put(self, req, resp):
            doc = self.json_body(req)
            docs.SecurityCliWhitelistDoc().validate(doc)

            full_doc = self.get_tve_config_dict()
            full_doc['json']['tve']['config']['security'] = {'cli-whitelist': doc}

            self.call_homecalling(resp, full_doc, 'security_cli_whitelist')

    class RemoteServers(res.BaseResource):
        @policy.enforce('config.remote-servers:get')
        def on_get(self, req, resp):
            resp.body = json_tools.get_json_value('//syslog/remote-servers',
                                                  res.UCPE_CONFIG_FILE)

        @policy.enforce('config.remote-servers:put')
        @falcon.before(res.BaseResource.max_body())
        def on_put(self, req, resp):
            doc = self.json_body(req)
            docs.RemoteServersDoc().validate(doc)

            full_doc = self.get_tve_config_dict()
            full_doc['json']['tve']['config']['syslog'] = {'remote-servers': doc}

            self.call_homecalling(resp, full_doc, 'remote-servers')

    class RemoteServer(res.BaseResource):
        @policy.enforce('config.remote-server:get')
        def on_get(self, req, resp, id):
            resp.body = json_tools.get_json_value_by_key(res.UCPE_CONFIG_FILE,
                                                         '//syslog/remote-servers/', 'id={}'.format(id))

    class SNMPServer(res.BaseResource):
        @policy.enforce('config.snmp-server:get')
        def on_get(self, req, resp):
            resp.body = json_tools.get_json_value('//snmp-server',
                                                  res.UCPE_CONFIG_FILE)

        @policy.enforce('config.snmp-server:put')
        @falcon.before(res.BaseResource.max_body())
        def on_put(self, req, resp):
            doc = self.json_body(req)
            docs.SNMPServerDoc().validate(doc)

            full_doc = self.get_tve_config_dict()
            full_doc['json']['tve']['config'] = {'snmp-server': doc}

            self.call_homecalling(resp, full_doc, 'snmp-server')

    class SysRes(res.BaseResource):
        @policy.enforce('config.system-resources:get')
        def on_get(self, req, resp):
            resp.body = json_tools.get_json_value('//system-resources',
                                                  res.UCPE_CONFIG_FILE)

        @policy.enforce('config.system-resources:put')
        @falcon.before(res.BaseResource.max_body())
        def on_put(self, req, resp):
            doc = self.json_body(req)
            docs.SysResDoc().validate(doc)

            full_doc = self.get_tve_config_dict()
            full_doc['json']['tve']['boot'] = {'system-resources': doc}

            self.call_homecalling(resp, full_doc, 'system-resources')

    class DhcpRelay(res.BaseResource):
        @policy.enforce('config.dhcp-relay:get')
        def on_get(self, req, resp):
            resp.body = json_tools.get_json_value('//dhcp-relay', res.UCPE_CONFIG_FILE)

        @policy.enforce('config.dhcp-relay:put')
        @falcon.before(res.BaseResource.max_body())
        def on_put(self, req, resp):
            doc = self.json_body(req)
            docs.DhcpRelayDoc().validate(doc)

            full_doc = self.get_tve_config_dict()
            full_doc['json']['tve']['config'] = {'dhcp-relay': doc}

            self.call_homecalling(resp, full_doc, 'dhcp-relay')

    class DhcpServer(res.BaseResource):
        @policy.enforce('config.dhcp-server:get')
        def on_get(self, req, resp):
            resp.body = json_tools.get_json_value('//dhcp-server', res.UCPE_CONFIG_FILE)

        @policy.enforce('config.dhcp-server:put')
        @falcon.before(res.BaseResource.max_body())
        def on_put(self, req, resp):
            doc = self.json_body(req)
            docs.DhcpServerDoc().validate(doc)

            full_doc = self.get_tve_config_dict()
            full_doc['json']['tve']['config'] = {'dhcp-server': doc}

            self.call_homecalling(resp, full_doc, 'dhcp-server')

    class ActivationScreenWizard(res.BaseResource):
        @policy.enforce('config.activation-screen-wizard:get')
        def on_get(self, req, resp):
            if os.path.isfile('/config/act-scr-wiz.json'):
                resp.body = json_tools.get_json_value('//activation-screen-wizard', '/config/act-scr-wiz.json')
            else:
                resp.body = {'enabled': 'false'}

        @policy.enforce('config.activation-screen-wizard:put')
        @falcon.before(res.BaseResource.max_body())
        def on_put(self, req, resp):
            doc = self.json_body(req)
            docs.ActivationScreenWizardDoc().validate(doc)

            full_doc = self.get_tve_config_dict()
            full_doc['json']['tve']['config'] = {'activation-screen-wizard': doc}

            self.call_homecalling(resp, full_doc, 'activation-screen-wizard')

    class ZtpAuthUpdate(res.BaseResource):
        # API tests:
        # get key file: curl -v -H "X-AUTH-TOKEN:$TOKEN" -H "Content-type: application/json" -k -X GET https://<ip_address>:7373/v1/config/ztp-auth-key
        # update key file: curl -v -H "X-AUTH-TOKEN:$TOKEN" -H "Content-type: application/json" -k -X PUT https://<ip_address>:7373/v1/config/ztp-auth-key -d '<string>'
        # upload key file : curl -v -H "X-AUTH-TOKEN:$TOKEN" -k -F"file=@<file_name>" https://<ip_address>:7373/v1/config/ztp-auth-key
        @policy.enforce('config.ztp-auth-key:get')
        def on_get(self, req, resp):
            if os.path.isfile(res.ZTP_AUTH_KEY):
                with open(res.ZTP_AUTH_KEY, 'r') as f:
                    resp.body = f.read()
                    f.close()
            else:
                self.set_status_400(resp, 'File does not exist')

        @policy.enforce('config.ztp-auth-key:put')
        def on_put(self, req, resp):
            data = req.stream.read()
            try:
                open(res.ZTP_AUTH_KEY, 'w').write(data)
            except Exception as ex:
                self.set_status_400(resp, 'Problem with token update. Error - %s' % ex)
                return

            resp.body = {'upload': 'ok'}
            resp.status = falcon.HTTP_201

        @policy.enforce('config.ztp-auth-key:post')
        def on_post(self, req, resp):
            input_file = req.get_param('file')
            data = input_file.file.read()
            fsize = int(len(data)) / (1024 * 1024)

            if input_file.filename:
                if fsize <= 1:
                    try:
                        open(res.ZTP_AUTH_KEY, 'w').write(data)
                    except Exception as ex:
                        self.set_status_400(resp, 'Problem with file upload. Error - %s' % ex)
                        return
                else:
                    resp.body = {'upload': 'failed', 'message': 'File size is greater than 1MB!'}
                    resp.status = falcon.HTTP_400
                    return

            resp.body = {'upload': 'ok'}
            resp.status = falcon.HTTP_201

    class KubernetesAdminConfig(res.BaseResource):
        @policy.enforce('config.kubernetes-admin-config:get')
        def on_get(self, req, resp):
            self.execute_command(resp, 'sudo cat /etc/kubernetes/admin.conf')
