#!/usr/bin/env python3
import json
import os
import subprocess
from collections import OrderedDict

import falcon
from oslo_log import log
from tveapi.common import resource as res
import sys
sys.path.insert(1, '/usr/lib/python3/dist-packages/tveapi/')
import send_notification

LOG = log.getLogger(__name__)

UCPE_STATUS_FILE = "/config/_hb/ucpe_status.json"
UCPE_STATUS_FILE_TEMPLATE = "{\"ucpe:status\": {\"status\": \"unknown\", \"hb-status\": \"unknown\", \"nsStatus\": \"unknown\", \"ztp-status\": \"unknown\", \"certs\": \"default\", \"license-id\": \"unknown\", \"apps\": false, \"gpu\": false, \"cluster\": {}, \"appslist\":{}}}"


class Reboot(res.BaseResource):
    def on_fail(self, resp, message):
        errmsg = OrderedDict()
        errmsg['404'] = ("{}".format(message))
        resp.body = errmsg
        resp.status = falcon.HTTP_404
        resp.data = json.dumps(errmsg, indent=4, separators=(',', ': ')).encode('utf8')
        resp.content_type = 'application/json'

    def on_get(self, req, resp):
        self.set_status_400(resp, "Only PUT supported.")
        return

    @falcon.before(send_notification.request_notifier)
    def on_put(self, req, resp):
        try:
            # no permisions for standard reading !!!
            # tveapi user in sudo group
            mngr_lines = subprocess.check_output(['/usr/bin/sudo', '/scripts/sysrepocfg.sh', '-X', '-x', '/ucpe:config/system:system/config-server', '-f', 'xml', '-d', 'running'], stderr=subprocess.STDOUT).decode('utf8')
            mngr_start = mngr_lines.index('<server>')
            mngr_end = mngr_lines.index('</server>')
            if mngr_start >= 0 and mngr_end >= 0:
                mng = mngr_lines[mngr_start + 8:mngr_end]
                subprocess.call(['/usr/bin/snmptrap', '-v', '2c', '-c', 'monitoring', '-r', '3', '-t', '7',
                                 '%s:1062' % mng, '', 'SNMPv2-MIB::warmStart'])

            subprocess.check_call(['/scripts/send_netconf_restart_notif', 'warm'])
            if os.path.exists('/config/_reboot/tve/now'):
                subprocess.call(['/usr/bin/sudo', 'rmdir', '/config/_reboot/tve/now'])

            # Create dirs
            subprocess.call(['/usr/bin/sudo', 'mkdir', '/config/_reboot/tve/now'])
        except subprocess.CalledProcessError as e:
            LOG.error("fail exception CalledProcessError")
            self.on_fail(resp, e)
            return
        except OSError as e:
            LOG.error("fail exception OSError")
            self.on_fail(resp, e)
            return

        return


class Apply_Config(res.BaseResource):
    def on_fail(self, resp, message):
        errmsg = OrderedDict()
        errmsg['404'] = ("{}".format(message))
        resp.body = errmsg
        resp.status = falcon.HTTP_404
        resp.data = json.dumps(errmsg, indent=4, separators=(',', ': ')).encode('utf8')
        resp.content_type = 'application/json'

    def on_get(self, req, resp):
        self.set_status_400(resp, "Only PUT supported.")
        return

    @falcon.before(send_notification.request_notifier)
    def on_put(self, req, resp):
        try:
            subprocess.check_call(['/usr/bin/sudo', 'systemctl', 'restart', 'homecalling.service'])
        except subprocess.CalledProcessError as e:
            LOG.error("fail exception CalledProcessError")
            self.on_fail(resp, e)
            return
        except OSError as e:
            LOG.error("fail exception OSError")
            self.on_fail(resp, e)
            return

        return


class Send_Config(res.BaseResource):
    def on_fail(self, resp, message):
        errmsg = OrderedDict()
        errmsg['404'] = ("{}".format(message))
        resp.body = errmsg
        resp.status = falcon.HTTP_404
        resp.data = json.dumps(errmsg, indent=4, separators=(',', ': ')).encode('utf8')
        resp.content_type = 'application/json'

    def on_get(self, req, resp):
        self.set_status_400(resp, "Only PUT supported.")
        return

    def on_put(self, req, resp):
        isNeedSchema = 0
        if req.content_length > 0:
            json_req = self.json_body(req)
            if 'isNeedSchema' in json_req:
                LOG.debug("isNeedSchema while body is {}".format(json_req['isNeedSchema']))
                isNeedSchema = json_req['isNeedSchema']

        try:
            if isNeedSchema == 1:
                subprocess.check_call(['/usr/bin/sudo', 'rm', '-rf', '/config/_sendconfig/schema/schema'])
                subprocess.check_call(['/usr/bin/sudo', 'mkdir', '-p', '/config/_sendconfig/schema/schema'])

            else:
                subprocess.check_call(['/usr/bin/sudo', 'rm', '-rf', '/config/_sendconfig/config/config'])
                subprocess.check_call(['/usr/bin/sudo', 'mkdir', '-p', '/config/_sendconfig/config/config'])

        except subprocess.CalledProcessError as e:
            LOG.error("fail exception CalledProcessError")
            self.on_fail(resp, e)
            return
        except OSError as e:
            LOG.error("fail exception OSError")
            self.on_fail(resp, e)
            return

        return


class NS_Status(res.BaseResource):
    def on_fail(self, resp, message):
        errmsg = OrderedDict()
        errmsg['404'] = ("{}".format(message))
        resp.body = errmsg
        resp.status = falcon.HTTP_404
        resp.data = json.dumps(errmsg, indent=4, separators=(',', ': ')).encode('utf8')
        resp.content_type = 'application/json'

    def on_get(self, req, resp):
        self.set_status_400(resp, "Only PUT supported.")
        return

    def update_status_file(self, key, value):
        if not (os.path.isfile(UCPE_STATUS_FILE)):
            ns_status_f = open(UCPE_STATUS_FILE, 'w')
            data = json.loads(UCPE_STATUS_FILE_TEMPLATE)
            data['ucpe:status'][key] = str(value)
            json.dump(data, ns_status_f)
            ns_status_f.close()
            os.chmod(UCPE_STATUS_FILE, 0o777)
        else:
            with open(UCPE_STATUS_FILE, "r+") as json_file:
                data = json.load(json_file)

                data['ucpe:status'][key] = str(value)

                json_file.seek(0)
                json.dump(data, json_file)
                json_file.truncate()
                json_file.close()

    def on_put(self, req, resp):
        ns_status = 'unknown'
        ns_supported = ['not available', 'not deployed', 'active', 'unknown',
                        'pending create', 'pending delete', 'pending update',
                        'pending scale in', 'pending scale out', 'inactive',
                        'down', 'dead', 'error', 'pending delete', 'pending create']
        data = req.stream.read()
        if data.lower() in ns_supported:
            ns_status = data.lower()
        else:
            LOG.error('ns_status "{}" not in supported list'.format(data))
            self.on_fail(resp, 'ns_status "{}" not in supported list'.format(data))
            return

        self.update_status_file('nsStatus', ns_status)
        return

#curl -H "X-Auth-Token: $TOKEN" -H "Content-type:application/json" -k --insecure  -X PUT https://10.3.76.87:7373/v1/monitoring -d '{"hostid":"myhost-12345"}'
class Monitoring(res.BaseResource):
    def on_fail(self, resp, message):
        errmsg = OrderedDict()
        errmsg['404'] = ("{}".format(message))
        resp.body = errmsg
        resp.status = falcon.HTTP_404
        resp.data = json.dumps(errmsg, indent=4, separators=(',', ': ')).encode('utf8')
        resp.content_type = 'application/json'

    def on_get(self, req, resp):
        hostid="Unknown"
        try:
            cmd='/usr/bin/sudo cat /config/collectd-hostname'
            stream = os.popen(cmd)
            hostid = stream.read().strip()
        except:
            pass
        resp.data = json.dumps({'hostid':'%s'% hostid} , indent=4, separators=(',', ': ')).encode('utf8')
        return

    def on_put(self, req, resp):
        hostid = ""
        if req.content_length > 0:
            json_req = self.json_body(req)
            if 'hostid' in json_req:
                hostid = json_req['hostid']
        if hostid == "":
            self.on_fail(resp, "Bad request")
            return
        try:
            os.system('/usr/bin/sudo bash -c "echo \"%s\"  > /config/collectd-hostname"' % hostid)
            os.system('/usr/bin/sudo bash -c \'sed -i "s|^Hostname .*|Hostname %s|"  /etc/collectd/collectd.conf\'' % hostid)
            os.system('/usr/bin/sudo bash -c \'sed -i "s|^#Hostname .*|Hostname %s|" /etc/collectd/collectd.conf\'' % hostid)
            os.system('/usr/bin/sudo bash -c "systemctl restart collectd.service"')
        except exception as e:
            LOG.error("fail exception CalledProcessError")
            self.on_fail(resp, e)
            return
        return


