#!/usr/bin/env python3
import os
import falcon
import json
import paramiko
import multiprocessing
from tveapi.common import resource as res
from tveapi.common import utils
from oslo_log import log
import tarfile
import random
import subprocess
import time


LOG = log.getLogger(__name__)
DISK_VOLUME_LOCATION="/var/lib/kubestore/"
DEFAULT_DISK_NAME="disk.img"


def check_for_req_params(params, required_params):
    """
    Check for required parameters
    """
    for param in required_params:
        if param not in params:
            return f'Required parameter missing: {required_params}'
    return


def get_status(status_file):
    """
    Get current saved status and if status
    is in 'done' or 'failed' state, send it
    only once and delete file.  
    """
    status = None
    if os.path.exists(status_file):
        with open(status_file, 'r') as f:
            status = f.read()
        f.close()
        if 'done' or 'failed' in status:
            os.remove(status_file)
    return status


def check_for_disk(disk):
    """
    Check for disk.img
    """
    for dir in os.walk(DISK_VOLUME_LOCATION):
        if disk in dir[0] and os.path.exists(os.path.join(dir[0], DEFAULT_DISK_NAME)):
            return dir[0]
    return


def get_tar_flag_from_algo(compression):
    algo = {
        'gzip': '-zcvf',
        'bzip2': '-jcvf',
        'xz': '-Jcvf',
    }
    return algo[compression]


class TarCommandBuilder(object):
    """
    Building a tar cmd command. To build command invoke method build.
    """

    DEFAULT_TEMPLATE = (
        "tar {0} {1} --absolute-names {2}")

    def __init__(self, filepath, tar_path, compression_algo='gzip'):
        self.tar_path = tar_path
        self.sparse = False
        self.sudo = False
        self.filepath = filepath
        self.compression_algo = get_tar_flag_from_algo(compression_algo)

    def set_sparse(self):
        """
        Records in the archive for the file where the holes (consecutive stretches of zeros) are, 
        and only archives the "real contents" of the file.
        """
        self.sparse = True

    def set_sudo(self):
        self.sudo = True

    def build(self):
        tar_command = self.DEFAULT_TEMPLATE.format(self.compression_algo, self.tar_path, self.filepath)

        if self.sparse:
            tar_command = "{0} {1}".format(tar_command, '--sparse')

        if self.sudo:
            tar_command = "{0} {1}".format('/usr/bin/sudo', tar_command)

        return tar_command


class SFTPSession:
    """
    Open FTP session and build archive.
    """
    def __init__(self, args, source_dir):
        self._ssh = paramiko.SSHClient()
        self._ftp = None
        self.args = args
        self.port = args['port'] if args['port'] != "" else 22
        self.source_dir = source_dir
        self.backup_file = os.path.join(res.TEMPORARY_DIR, args['backup'] + '.tar')
        self.remote_file = os.path.join(args['path'], args['backup']+'.tar')
        self.transferred_bytes = 0
        self.type = 'backup'
        self.current_status = None
        self.status_file = ''
        

    def connection(self):
        LOG.debug("-- SFTPSession: connection --")
        if not self.is_connected():
            LOG.debug("-- SFTPSession: try to connect --")
            self._ssh.load_system_host_keys()
            self._ssh.set_missing_host_key_policy(paramiko.client.AutoAddPolicy())
            self._ssh.connect(self.args['host'], port=self.port,
                            username=self.args['user'], password=self.args['password'], timeout=5, look_for_keys=True)
            self._ftp = self._ssh.open_sftp()

    def is_connected(self):
        LOG.debug("-- SFTPSession: is_connected --")
        transport = self._ssh.get_transport() 
        return transport if transport == None else transport.is_active()

    def set_status(self, status, transferred=0, remaining=0):
        if self.current_status is not status:
            self.current_status = status
        # Set status on every 10 transferred MBs
        elif self.transferred_bytes + 2**20*10 <= transferred or remaining:
            self.transferred_bytes = transferred
        else:
            return

        state = json.dumps({self.type: status, 'size': transferred})
        with open(self.status_file, 'w') as sf:
            print(state, file=sf, end='')
        sf.close()

    def write_status(self, transferred, remaining):
        # Get transferred bytes and because of race condition, do not call set_status -> done,
        # before tarfile method finalize extracting of backup image file.
        if transferred == remaining and self.type != "restore":
            self.set_status('done', transferred, remaining)
        else:
            self.set_status('in progress', transferred)

    def backup(self):
        LOG.debug("-- SFTPSession: backup --")

        self.set_status('in progress')
        # Maybe this will be slow operation (CPU and HDD overload)
        # TODO Read in memory and send chunks on remote
        
        image_path = os.path.join(self.source_dir, DEFAULT_DISK_NAME)
        # with tarfile.open(self.backup_file, "w:gz", ignore_zeros=True) as tar:
        #    tar.add(image_path, arcname=os.path.basename(image_path), recursive=False)
        #    tar.close()
        tar_command = TarCommandBuilder(
                image_path,
                self.backup_file)
        tar_command.set_sparse()
        tar_command.set_sudo()
        command = tar_command.build()
        code, _, stderr = utils.execute_shell_command(command, quiet=True)

        # 'file changed as we read it' this output  means that some files were changed
        # while being archived and so the resulting archive does not contain the exact
        # copy of the file set.
        if code != 0 and 'file changed as we read it' not in str(stderr):
            raise IOError('Unable to backup disk.img')
        self._ftp.put(self.backup_file, self.remote_file, callback=self.write_status)

    def restore(self):
        LOG.debug("-- SFTPSession: restore --")
        self.set_status('in progress')
        # Maybe this will be slow operation (CPU and HDD overload)
        # TODO Read in memory and send chunks on remote
        
        self._ftp.get(self.remote_file, self.backup_file, callback=self.write_status)
        os.chdir(self.source_dir)
        code, _, _ = utils.execute_shell_command("/usr/bin/sudo chown tveapi:www-data disk.img")
        if code != 0:
            raise IOError('Unable to restore disk.img')
        with tarfile.open(self.backup_file, "r:gz", ignore_zeros=True) as tar:
            for member in tar.getmembers():
                if member.isreg():
                    member.name = os.path.basename(member.name)
                    tar.extract(member,self.source_dir)
                    tar.close()
        self.set_status('done', self.transferred_bytes)

    def clear(self):
        LOG.debug("-- SFTPSession: remove --")
        try:
            if os.path.exists(self.backup_file):
                utils.execute_shell_command("/usr/bin/sudo chown tveapi:www-data " + self.backup_file)
                os.remove(self.backup_file)
        except Exception:
            pass

    def close(self):
        LOG.debug("-- SFTPSession: close --")
        try:
            self._ssh.close()
            self._ftp.close()
        except Exception:
            pass


def backup_restore(args, disk_dir):
    session = SFTPSession(args, disk_dir)
    session.type = multiprocessing.current_process().name
    try:
        session.connection()
        if session.type == 'backup':
            session.status_file = os.path.join(res.TEMPORARY_DIR, args['disk'] + '_backup_status')
            session.backup()
        elif session.type == 'restore':
            session.status_file = os.path.join(res.TEMPORARY_DIR, args['backup'] + '_restore_status')
            session.restore()
    except paramiko.ssh_exception.NoValidConnectionsError as e:
        session.set_status(f'failed - connection error - {e.strerror}')
    except paramiko.ssh_exception.AuthenticationException as e:
        session.set_status(f'failed - authentication error - {e}')
    except OSError as e:
        if e.errno == 2:
            session.set_status(f'failed - connection error - No such file or directory')
        else:
            session.set_status(f'failed - connection error - {e}')
    except Exception as e:
        session.set_status(f'failed - error - {e}')
    session.clear()
    session.close()
    del session


class Volumes(object):
    """
    Volumes class contains all logic about backup and restore feature for Kubernetes Pods and Kubevirt VMs.

    How to test:
    
    Get API TOKEN:
    OS_USERNAME, OS_PASSWORD for the internal system user. See local-openrc.env
    Get TOKEN : TOKEN=$(curl -s -k -d '{"user":"OS_USERNAME","password":"OS_PASSWORD"}' -H "Content-type: application/json" -X GET https://127.0.0.1:17374/v3/auth/tokens)

    API tests:
    Create VM DataVolume backup: curl -s -H "X-Auth-Token: $TOKEN" --insecure -H "Content-Type: application/json" -d \
                    '{
                        "disk": "fedoravs-dv3",
                        "backup": "fedoravs-dv3-backup-202311291646",
                        "path": "/var/backups/",
                        "host": "1.2.3.4",
                        "port": "22",
                        "user":"admin",
                        "password":"very-secure-password"
                    }' \
                    -X PUT \
                    https://127.0.0.1:7373/v1/kubernetes/backup

    Get VM DataVolume status for backup: curl -s -H "X-Auth-Token: $TOKEN" --insecure -H "Content-Type: application/json" -d \
                    '{
                        "disk": "fedoravs-dv3"
                    }' \
                    -X GET \
                    https://127.0.0.1:7373/v1/kubernetes/backup

    Delete VM DataVolume backup: curl -s -H "X-Auth-Token: $TOKEN" --insecure -H "Content-Type: application/json" -d \
                    '{
                        "backup": "fedoravs-dv3-backup-202311291646",
                        "path": "/var/backups/",
                        "host": "1.2.3.4",
                        "port": "22",
                        "user":"admin",
                        "password":"very-secure-password"
                    }' \
                    -X DELETE \
                    https://127.0.0.1:7373/v1/kubernetes/backup

    Restore VM DataVolume: curl -s -H "X-Auth-Token: $TOKEN" --insecure -H "Content-Type: application/json" -d \
                    '{
                        //params like backup,
                        "disk": "fedoravs-dv3-<...>",
                        "target": "vm"
                    }' \
                    -X PUT \
                    https://127.0.0.1:7373/v1/kubernetes/restore

    Create POD backup: curl -s -H "X-Auth-Token: $TOKEN" --insecure -H "Content-Type: application/json" -d \
                        '{
                            "backup": "alpine-backup-202311291646",
                            "path": "/var/backups/",
                            "host": "1.2.3.4",
                            "port": "22",
                            "user": "admin",
                            "password": "very-secure-password",
                            "pods": [{
                                        "name": "alpine",
                                        "mountpoints": [{
                                                          "name": "alpine-storage",
                                                          "mountPath": "/usr/share/alpine"
                                                       }]
                                   }]}' \
                         -X PUT \
                         https://127.0.0.1:7373/v1/kubernetes/backup

    Get status for POD backup: curl -s -H "X-Auth-Token: $TOKEN" --insecure -H "Content-Type: application/json" -d \
                    '{
                        "id": "12345"
                    }' \
                    -X GET https://127.0.0.1:7373/v1/kubernetes/backup

    Delete POD backup: curl -s -H "X-Auth-Token: $TOKEN" --insecure -H "Content-Type: application/json" -d \
                    '{
                        "backup": "alpine-backup-202311291646",
                        "path": "/var/backups/",
                        "host": "1.2.3.4",
                        "port": "22",
                        "user": "admin",
                        "password": "very-secure-password"
                    }' \
                    -X DELETE \
                    https://127.0.0.1:7373/v1/kubernetes/backup

    Restore POD: curl -s -H "X-Auth-Token: $TOKEN" --insecure -H "Content-Type: application/json" -d \
                    '{
                        "backup": "fedoravs-dv3-backup-202311291646",
                        "path": "/var/backups/",
                        "host": "1.2.3.4",
                        "port": "22",
                        "user": "admin",
                        "password": "very-secure-password",
                        "target": "pod"
                    }' \
                    -X PUT \
                    https://127.0.0.1:7373/v1/kubernetes/restore
    """


    class Backup(res.BaseResource):
        def on_put(self, req, resp):

            try:
                json_req = self.json_body(req)
            except:
                self.on_fail(resp, 'Invalid JSON data format')
                return

            if 'pods' in json_req:
                # Check for required parameters
                required_params = ['pods', 'backup', 'path', 'host', 'user', 'password', 'port']
                ret = check_for_req_params(json_req, required_params)
                if ret:
                    self.on_fail(resp, ret)
                    return
                cookie = random.randint(500, 50000)
                command_subprocess_starter = multiprocessing.Process(target=backup_pods, args=(json_req, cookie,))
                command_subprocess_starter.daemon = True
                command_subprocess_starter.start()
                resp.body = json.dumps({'backup': 'starting', 'size': 0, 'id': cookie}, indent=4, separators=(',', ': ')).encode('utf8')
                resp.status = falcon.HTTP_202
                return

            if 'disk' in json_req:
                # Check for required parameters
                required_params = ['disk', 'backup', 'path', 'host', 'user', 'password', 'port']
                ret = check_for_req_params(json_req, required_params)
                if ret:
                    self.on_fail(resp, ret)
                    return

                # Check is disk.img located on disk
                disk_dir = check_for_disk(json_req['disk'])
                if not disk_dir:
                    self.on_fail(resp, f"{json_req['disk']} VM's disk is missing on filesystem!")
                    return

                command_subprocess_starter = multiprocessing.Process(target=backup_restore, name='backup', args=(json_req,disk_dir))
                command_subprocess_starter.daemon = True
                command_subprocess_starter.start()

                resp.status = falcon.HTTP_202
                resp.body = {'backup': 'starting', 'size': 0}


        def on_delete(self, req, resp):

            try:
                json_req = self.json_body(req)
            except:
                self.on_fail(resp, 'Invalid JSON data format')
                return

            # Check for required parameters
            required_params = ['backup', 'path', 'host', 'user', 'password', 'port']
            ret = check_for_req_params(json_req, required_params)
            if ret:
                self.on_fail(resp, ret)
                return
            
            try:
                clear_session = SFTPSession(json_req, "")
                clear_session.connection()

                # Check if file is located on remote
                listdir = clear_session._ftp.listdir(json_req['path'])

                if json_req['backup'] + '.tar' in listdir:
                    clear_session._ftp.remove(os.path.join(json_req['path'], json_req['backup'] + '.tar'))
                    resp.body = {'delete': 'ok'}
                elif json_req['backup'] + '.tgz' in listdir:
                    clear_session._ftp.remove(os.path.join(json_req['path'], json_req['backup'] + '.tgz'))
                    resp.body = {'delete': 'ok'}
                else:
                    raise Exception('No such file or directory')
            except paramiko.ssh_exception.NoValidConnectionsError as e:
                resp.body = {'delete': 'failed - connection error - ' + f'{e.strerror}'}
            except paramiko.ssh_exception.AuthenticationException as e:
                resp.body = {'delete': 'failed - authentication error - ' + f'{e}'}
            except OSError as e:
                if e.errno == 2:
                    resp.body = {'delete': 'failed - connection error - No such file or directory'}
                else:
                    resp.body = {'delete': 'failed - connection error - ' + f'{e}'}
            except Exception as e:
                resp.body = {'delete': 'failed - error - ' + f'{e}'}

            clear_session.close()
            resp.status = falcon.HTTP_200


        def on_get(self, req, resp):

            try:
                json_req = self.json_body(req)
            except:
                self.on_fail(resp, 'Invalid JSON data format')
                return

            # Check for required parameters
            # Get status
            if req.content_length is not None:
                if 'disk' in json_req:
                    clean_old(res.TEMPORARY_DIR)

                    status_file = os.path.join(res.TEMPORARY_DIR, json_req['disk'] + '_backup_status')
                    if os.path.exists(status_file):
                        with open(status_file, 'r') as f:
                            status = f.read()
                        resp.body = json.loads(status)
                        resp.status = falcon.HTTP_200
                    else:
                        self.on_fail(resp, f'Missing status for {json_req["disk"]}.')
                    return
                elif 'id' in json_req:
                    clean_old()

                    filename = False
                    outdir = '/var/log/tveapi/{}'.format(json_req['id'])
                    status = 'unknown'
                    stats = 0
                    resp.status = falcon.HTTP_200
                    try:
                        with open(os.path.join(outdir,'status.txt'), 'r') as statusfile:
                            status = statusfile.readline()
                        with open(os.path.join(outdir,'backup.txt'), 'r') as bfile:
                            filename = os.path.join(outdir, bfile.readline())
                    except:
                        resp.body = {'backup': status, 'size': stats}
                        return

                    if filename and os.path.exists(filename):
                        stats = os.stat(filename).st_size
                    else:
                        status = 'failed'
                    resp.body = {'backup': status, 'size': stats}
                    return

            required_params = ['disk', 'id']
            ret = check_for_req_params(json_req, required_params)
            self.on_fail(resp, ret)
            return


    class Restore(res.BaseResource):
        def on_put(self, req, resp):

            try:
                json_req = self.json_body(req)
            except:
                self.on_fail(resp, 'Invalid JSON data format')
                return

            # Check for required parameters
            required_params = ['backup', 'path', 'host', 'user', 'password', 'port', 'target']
            ret = check_for_req_params(json_req, required_params)
            if ret:
                self.on_fail(resp, ret)
                return

            if 'vm' in json_req['target']:
                if check_for_req_params(json_req, ['disk']):
                    self.on_fail(resp, 'Required parameter missing: \'disk\'')
                    return
                # Check is disk.img located on disk
                disk_dir = check_for_disk(json_req['disk'])
                if not disk_dir:
                    self.on_fail(resp, f"{json_req['disk']} VM's disk is missing on filesystem!")
                    return

                command_subprocess_starter = multiprocessing.Process(target=backup_restore, name='restore', args=(json_req,disk_dir))
                command_subprocess_starter.daemon = True
                command_subprocess_starter.start()

                resp.body = {'restore': 'starting', 'size': 0}
            elif 'pod' in json_req['target']:
                cookie = random.randint(500, 50000)
                command_subprocess_starter = multiprocessing.Process(target=restore_pods, args=(json_req, cookie,))
                command_subprocess_starter.daemon = True
                command_subprocess_starter.start()
                resp.body = json.dumps({'restore': 'starting', 'size': 0, 'id': cookie}, indent=4, separators=(',', ': ')).encode('utf8')

            resp.status = falcon.HTTP_202


        def on_get(self, req, resp):

            try:
                json_req = self.json_body(req)
            except:
                self.on_fail(resp, 'Invalid JSON data format')
                return

            if 'id' in json_req:
                clean_old()

                if req.content_length is not None:
                    resp.status = falcon.HTTP_200
                    filename = False
                    outdir = '/var/log/tveapi/{}'.format(json_req['id'])
                    status = 'unknown'
                    stats = 0
                    with open(os.path.join(outdir,'status.txt'), 'r') as statusfile:
                        status = statusfile.readline()
                    with open(os.path.join(outdir,'backup.txt'), 'r') as bfile:
                        filename = os.path.join(outdir,bfile.readline())

                    if filename and os.path.exists(filename):
                         stats = os.stat(filename).st_size
                    else:
                         status = 'failed'
                    resp.body = {'restore': status, 'size': stats}
                    return
            else:
                clean_old(res.TEMPORARY_DIR)

                # Check for required parameters
                required_params = ['backup']
                ret = check_for_req_params(json_req, required_params)
                if ret:
                    self.on_fail(resp, ret)
                    return

                # Get status
                if req.content_length is not None:
                    status_file = os.path.join(res.TEMPORARY_DIR, json_req['backup'] + '_restore_status')
                    if os.path.exists(status_file):
                        with open(status_file, 'r') as f:
                            status = f.read()
                        resp.body = json.loads(status)
                        resp.status = falcon.HTTP_200
                    else:
                        self.on_fail(resp, f'Missing status for {json_req["backup"]}.')
                    return


def backup_pods(json_data=None, cookie=None):
    outdir = '/var/log/tveapi/{}'.format(cookie)
    basef = os.path.join(outdir,'backup.txt')
    localf = os.path.join(outdir,json_data['backup']+'.tgz')
    remotef = os.path.join(json_data['path'],json_data['backup']+'.tgz')
    statusf = os.path.join(outdir,'status.txt')
    manifestf = os.path.join(outdir,'MANIFEST')
    containers = []
    backedup = []
    if not os.path.exists(outdir):
        os.makedirs(outdir, exist_ok=True)
    with open(statusf, 'w+') as statusfile:
        if json_data is None:
            statusfile.write('failed')
            return
        statusfile.write('in progress')

    with open(basef, 'w+') as statusfile:
        statusfile.write(json_data['backup']+'.tgz')
    tarf = tarfile.open(localf,'w:gz')
    find_pod = False
    for pod in json_data['pods']:
        backedup.append('\n['+pod['name']+']')
        output = subprocess.check_output("sudo crictl ps", shell=True).decode().split('\n')
        for line in output:
            if pod['name'] in line:
                containers.append(json.loads(subprocess.check_output("sudo crictl inspect {}".format(line.split()[0]), shell=True)))
                find_pod = True
        if not find_pod:
            with open(statusf, 'w+') as statusfile:
                statusfile.write(f'failed - pod not found - {pod["name"]}')
                print(f'{json_data["pods"]}, {output}')
            tarf.close()
            return
        for mount in pod['mountpoints']:
            for bmount in containers:
                tobackup = next((item for item in bmount['status']['mounts'] if item['containerPath'] == mount['mountPath']),None)
                if tobackup is not None:
                    tarf.add(tobackup['hostPath'])
                    backedup.append(tobackup['hostPath'])
                    break

    getnames = tarf.getnames()
    with open(manifestf, 'w+') as manifest:
        manifest.writelines(i+'\n' for i in backedup)
        manifest.writelines('\nFiles:\n')
        manifest.writelines(i+'\n' for i in getnames)

    tarf.add(manifestf, recursive=False)
    tarf.close()

    try:
        client = paramiko.SSHClient()
        client.load_system_host_keys()
        client.set_missing_host_key_policy(paramiko.client.AutoAddPolicy())
        client.connect(json_data['host'], port=json_data['port'], username=json_data['user'], password=json_data['password'], look_for_keys=False)
    except OSError as e:
        with open(statusf, 'w+') as statusfile:
            statusfile.write(f'failed - connection error - {e}')
        client.close()
        return
    except paramiko.ssh_exception.AuthenticationException as e:
        with open(statusf, 'w+') as statusfile:
            statusfile.write(f'failed - authentication error - {e}')
        client.close()
        return

    try:
        sftp_client = client.open_sftp()
        sftp_client.put(localf, remotef) # pass callback to get bytes transferred so far and the total bytes to be transferred
        sftp_client.close()
        client.close()
    except OSError as e:
        with open(statusf, 'w+') as statusfile:
            statusfile.write('failed - unable to open file=' + remotef +f' - {e}')
        sftp_client.close()
        client.close()
        return

    with open(statusf, 'w+') as statusfile:
        statusfile.write('done')


def restore_pods(json_data=None, cookie=None):
    outdir = '/var/log/tveapi/{}'.format(cookie)
    basef = os.path.join(outdir,'backup.txt')
    localf = os.path.join(outdir,json_data['backup']+'.tgz')
    remotef = os.path.join(json_data['path'],json_data['backup']+'.tgz')
    statusf = os.path.join(outdir,'status.txt')
    containers = []
    backedup = []
    if not os.path.exists(outdir):
        os.makedirs(outdir, exist_ok=True)
    with open(statusf, 'w+') as statusfile:
        if json_data is None:
            statusfile.write('failed')
            return
        statusfile.write('in progress')
    with open(basef, 'w+') as statusfile:
        statusfile.write(json_data['backup']+'.tgz')

    try:
        client = paramiko.SSHClient()
        client.load_system_host_keys()
        client.set_missing_host_key_policy(paramiko.client.AutoAddPolicy())
        client.connect(json_data['host'], port=json_data['port'], username=json_data['user'], password=json_data['password'], look_for_keys=False)
    except OSError as e:
        with open(statusf, 'w+') as statusfile:
            statusfile.write(f'failed - connection error - {e}')
        client.close()
        return
    except paramiko.ssh_exception.AuthenticationException as e:
        with open(statusf, 'w+') as statusfile:
            statusfile.write(f'failed - authentication error - {e}')
        client.close()
        return

    try:
        sftp_client = client.open_sftp()
        sftp_client.get(remotef, localf) # pass callback to get bytes transferred so far and the total bytes to be transferred
        sftp_client.close()
        client.close()
    except OSError as e:
        with open(statusf, 'w+') as statusfile:
            statusfile.write('failed - unable to open file=' + remotef + f' - {e}')
        sftp_client.close()
        client.close()
        return

    output = subprocess.check_output('sudo tar xzf ' + localf + ' -C /', shell=True)

    with open(statusf, 'w+') as statusfile:
        statusfile.write('done')


def clean_old(root_dir='/var/log/tveapi/'):
    """
    Clean status files older than 6 hours
    """
    for filename in os.listdir(root_dir):
        workpath = os.path.join(root_dir, filename)
        if root_dir == res.TEMPORARY_DIR and any(x in workpath for x in ['_backup_status', '_restore_status']):
            diffs = int(time.time()) - int(os.path.getmtime(workpath))
            if diffs > 21600:
                os.remove(workpath)
        elif os.path.isdir(workpath):
            workfile = os.path.join(workpath, 'status.txt')
            if os.path.isfile(workfile):
                diffs = int(time.time()) - int(os.path.getmtime(workfile))
                if diffs > 21600:
                    # files older that 6 hours, delete
                    for root, dirs, files in os.walk(workpath):
                        for f in files:
                            os.remove(os.path.join(root, f))
                    os.rmdir(workpath)
