#!/usr/bin/env python3
import json
import subprocess

import falcon
from oslo_log import log
from tveapi import policy
from tveapi.common import resource as res
from tveapi.common.utils import check_connectivity
from tveapi.json_tools import json_tools
import sys
sys.path.insert(1, '/usr/lib/python3/dist-packages/tveapi/')

LOG = log.getLogger(__name__)

request_json_err_msg = 'This API only supports requests encoded as JSON.'
response_json_err_msg = 'This API only supports responses encoded as JSON.'


class Enum(set):
    def __getattr__(self, name):
        if name in self:
            return name
        raise AttributeError


TVE_CONNECTIVITY_STATUS = Enum(["reachable", "unreachable"])
TVE_IP = '169.254.0.1'


class TVE(object):
    class ImageServer(res.BaseResource):
        ##@policy.enforce('status.image-server:get')
        def on_get(self, req, resp):
            return_code = check_connectivity('ucpe-manager')
            if return_code:
                resp.body = TVE_CONNECTIVITY_STATUS.reachable
            else:
                resp.body = TVE_CONNECTIVITY_STATUS.unreachable

    class Homecalling(res.BaseResource):
        ##@policy.enforce('status.homecalling:get')
        def on_get(self, req, resp):
            self.execute_command(resp, 'cat /proc/cmdline')

    class NetworkingRoutes(res.BaseResource):
        ##@policy.enforce('status.networking-routes:get')
        def on_get(self, req, resp):
            self.execute_command(resp, 'ip route show')

    class NetworkingInterfaces(res.BaseResource):
        ##@policy.enforce('status.networking-interfaces:get')
        def on_get(self, req, resp):
            self.execute_command(resp, 'ip -s a show')

    class NTP(res.BaseResource):
        # @policy.enforce('status.ntp:get')
        def on_get(self, req, resp):
            self.execute_command(resp, 'timedatectl')

    class ZTP(res.BaseResource):
        def on_get(self, req, resp):
            self.execute_command(resp, 'cat /config/_hb/last_known_err')

    class RoutingStatic(res.BaseResource):
        # @policy.enforce('status.routing-static:get')
        def on_get(self, req, resp):
            self.execute_command(resp, 'ip route show | grep \'zeb\|sta\' || true ')

    class Hardware(res.BaseResource):
        # @policy.enforce('status.hardware:get')
        def on_get(self, req, resp):
            self.execute_command(resp, 'sudo dmidecode')
            if resp.status == falcon.HTTP_INTERNAL_SERVER_ERROR and "entry point missing" in resp.body['err_msg']:
                resp.status = falcon.HTTP_200
                resp.body = "not supported"

    class CPU(res.BaseResource):
        @policy.enforce('status.cpu:get')
        def on_get(self, req, resp):
            self.execute_command(resp, 'sudo dmidecode -s processor-version')
            if resp.status == falcon.HTTP_INTERNAL_SERVER_ERROR and "entry point missing" in resp.body['err_msg']:
                resp.status = falcon.HTTP_200
                resp.body = "not supported"

    class System(res.BaseResource):
        @policy.enforce('status.system:get')
        def on_get(self, req, resp):
            self.execute_command(resp, 'systemctl status | grep State \
                                                             | grep -v grep')
            arr = resp.body.split(':')
            resp.body = {'status': arr[1].strip()}

    class UCPE(res.BaseResource):
        @policy.enforce('status.ucpe:get')
        def on_get(self, req, resp):
            if not req._params:
                try:
                    resp.body = json_tools.get_json_value('/ucpe:status/',
                                                          res.UCPE_STATUS_FILE) + "\n"
                except Exception as ex:
                    self.set_status_400(resp, 'No information about uCPE status - %s' % ex)
                    return
            else:
                if len(req._params) > 1:
                    self.set_status_400(resp, 'Only one search parameter is supported')
                    return
                try:
                    for key, value in req._params.items():
                        if key == 'status':
                            resp.body = json_tools.get_json_value('/ucpe:status/{}'.format(value),
                                                                  res.UCPE_STATUS_FILE) + "\n"
                except Exception as ex:
                    self.set_status_400(resp, 'No information about uCPE status - %s' % ex)
                    return

    class VMSize(res.BaseResource):
        @policy.enforce('status.system:get')
        def on_get(self, req, resp):
            vm_size = vm_size + int(subprocess.check_output(['sudo', 'du', '-s', '-b', '/var/lib/libvirt']).split()[0])
            vm_size = vm_size + int(subprocess.check_output(['sudo', 'du', '-s', '-b', '/var/lib/mysql']).split()[0])

            resp.body = {'vm-size': vm_size}

    class BackupRestoreCurrent(res.BaseResource):
        def on_get(self, req, resp):
            try:
                with open('/run/br-current', 'r') as f:
                    myb = json.load(f)

                LOG.critical('tveapi br-current myb {}'.format(myb))

                if "completed" == myb['status']:
                    resp.body = myb
                    return

                command = "sudo /scripts/parseprogress.sh percent"
                subprocess.check_output(command, shell=True)

                with open('/run/br-percent', 'r') as f:
                    br_percent = f.read()

                command = "sudo /scripts/parseprogress.sh eta"
                subprocess.check_output(command, shell=True)

                with open('/run/br-eta', 'r') as f:
                    br_eta = f.read().strip()

                br_eta_hour = int(br_eta) / 3600
                br_eta_min = int(br_eta) / 60
                br_eta_sec = int(br_eta) % 60
                resp.body = {'status': myb['status'], 'job-id': str(myb['job-id']), 'percent-complete': int(br_percent),
                             'eta': '{:02d}:{:02d}:{:02d}'.format(br_eta_hour, br_eta_min, br_eta_sec)}
            except Exception as e:
                LOG.error('tveapi br-current : {}'.format(e))
                resp.body = {'status': 'unknown', 'job-id': 0, 'percent-complete': 0, 'eta': '0'}
                pass

            return
